# -*- coding:utf-8 -*-
from abc import abstractmethod
from pathlib import Path
from threading import Lock
from typing import Union, Any, Optional, Callable

from .setter import OriginalSetter, BaseSetter


class OriginalRecorder(object):
    """记录器的基类"""
    _SUPPORTS: tuple = ...
    _cache: int = ...
    _path: Optional[str] = ...
    _type: Optional[str] = ...
    _data: list = ...
    _lock: Lock = ...
    _pause_add: bool = ...
    _pause_write: bool = ...
    _setter: Optional[OriginalSetter] = ...
    _data_count: int = ...
    _file_exists: bool = ...
    _backup_path: str = ...
    _backup_times: int = ...
    _backup_interval: int = ...
    _backup_new_name: bool = ...
    show_msg: bool = ...

    def __init__(self,
                 path: Union[str, Path, None] = None,
                 cache_size: int = None) -> None:
        """
        :param path: 保存的文件路径
        :param cache_size: 每接收多少条记录写入文件，0为不自动写入
        """
        ...

    def __del__(self) -> None:
        """对象关闭时把剩下的数据写入文件"""
        ...

    @property
    def set(self) -> OriginalSetter:
        """返回用于设置属性的对象"""
        ...

    @property
    def cache_size(self) -> int:
        """返回缓存大小"""
        ...

    @property
    def path(self) -> str:
        """返回文件路径"""
        ...

    @property
    def type(self) -> str:
        """返回文件类型"""
        ...

    @property
    def data(self) -> Union[dict, list]:
        """返回当前保存在缓存的数据"""
        ...

    def record(self) -> str:
        """记录数据，返回文件路径"""
        ...

    def clear(self) -> None:
        """清空缓存中的数据"""
        ...

    def backup(self,
               path: Union[str, Path] = 'backup',
               name: str = None) -> None:
        """把当前文件备份到指定路径
        :param path: 文件夹路径
        :param name: 保存的文件名，为None使用记录目标指定的文件名
        """
        ...

    @abstractmethod
    def add_data(self, data):
        ...

    @abstractmethod
    def _record(self):
        ...


class BaseRecorder(OriginalRecorder):
    """Recorder和DBRecorder的父类"""
    _SUPPORTS: tuple = ...
    _encoding: str = ...
    _before: Optional[list] = ...
    _after: Optional[list] = ...
    _table: Optional[str] = ...
    _setter: BaseSetter = ...
    _make_final_data: Callable = ...
    _auto_new_header: bool = ...

    def __init__(self, path: Union[None, str, Path] = None, cache_size: int = None) -> None:
        """
        :param path: 保存的文件路径
        :param cache_size: 每接收多少条记录写入文件，0为不自动写入
        """
        ...

    @property
    def set(self) -> BaseSetter:
        """返回用于设置属性的对象"""
        ...

    @property
    def before(self) -> Union[dict, list]:
        """返回当前before内容"""
        ...

    @property
    def after(self) -> Union[dict, list]:
        """返回当前after内容"""
        ...

    @property
    def table(self) -> Optional[str]:
        """返回当前使用的表名"""
        ...

    @property
    def tables(self) -> list:
        """返回所有表名"""
        ...

    @property
    def encoding(self) -> str:
        """返回csv文件使用的编码格式"""
        ...

    @abstractmethod
    def _record(self):
        ...



# -*- coding:utf-8 -*-
from pathlib import Path
from typing import Union, Optional

from .base import OriginalRecorder


class ByteRecorder(OriginalRecorder):
    _SUPPORTS: tuple = ...
    __END: tuple = ...
    data: list = ...

    def __init__(self,
                 path: Union[None, str, Path] = None,
                 cache_size: int = 1000):
        """用于记录字节数据的工具
        :param path: 保存的文件路径
        :param cache_size: 每接收多少条记录写入文件，0为不自动写入
        """
        ...

    def add_data(self,
                 data: bytes,
                 seek: int = None) -> None:
        """添加一段二进制数据
        :param data: bytes类型数据
        :param seek: 在文件中的位置，None表示最后
        :return: None
        """
        ...

    def _record(self) -> None:
        """记录数据到文件"""
        ...



# -*- coding:utf-8 -*-
from pathlib import Path
from typing import Union, Any, Optional, List, Tuple, Dict

from .base import OriginalRecorder, BaseRecorder
from .cell_style import CellStyle
from .db_recorder import DBRecorder
from .recorder import Recorder
from .tools import Header


class OriginalSetter(object):
    _recorder: OriginalRecorder = ...

    def __init__(self, recorder: OriginalRecorder): ...

    def cache_size(self, size: int) -> OriginalSetter:
        """设置缓存大小
        :param size: 缓存大小
        :return: 设置对象自己
        """
        ...

    def path(self, path: Union[str, Path]) -> OriginalSetter:
        """设置文件路径
        :param path: 文件路径
        :return: 设置对象自己
        """
        ...

    def show_msg(self, on_off: bool) -> OriginalSetter:
        """设置是否显示运行信息
        :param on_off: bool表示开关
        :return: 设置对象自己
        """
        ...

    def auto_backup(self,
                    interval: int = None,
                    path: Union[str, Path] = None,
                    new_name: bool = None) -> OriginalSetter:
        """设置自动备份相关参数
        :param interval: 自动保存多少次时触发备份，为0表示不自动备份，为None时不修改已设置值（初始为0）
        :param path: 备份文件存放路径，为None时不修改已设置值（初始为 'backup'）
        :param new_name: 是否用新名称备份，为None时不修改已设置值（初始为True）
        :return: 设置对象自己
        """
        ...


class BaseSetter(OriginalSetter):
    _recorder: BaseRecorder = ...

    def table(self, name: Union[str, bool]) -> BaseSetter:
        """设置默认表名
        :param name: 表名
        :return: 设置对象自己
        """
        ...

    def auto_new_header(self, on_off: bool = True) -> BaseSetter:
        """数据中有表头不存在的列时是否自动添加到表头，只有xlsx和csv格式有效
        :param on_off: bool表示开关
        :return: 设置对象自己
        """
        ...

    def before(self, data: Any) -> BaseSetter:
        """设置在数据前面补充的列
        :param data: 列表、元组或字符串，为字符串时则补充一列
        :return: 设置对象自己
        """
        ...

    def after(self, data: Any) -> BaseSetter:
        """设置在数据后面补充的列
        :param data: 列表、元组或字符串，为字符串时则补充一列
        :return: 设置对象自己
        """
        ...

    def _set_after_before(self, before: bool, data: Any) -> BaseSetter: ...


class RecorderSetter(BaseSetter):
    _recorder: Recorder = ...

    def __init__(self, recorder: Recorder): ...

    # -------------------上级开始-------------------
    def cache_size(self, size: int) -> RecorderSetter:
        """设置缓存大小
        :param size: 缓存大小
        :return: 设置对象自己
        """
        ...

    def show_msg(self, on_off: bool) -> RecorderSetter:
        """设置是否显示运行信息
        :param on_off: bool表示开关
        :return: 设置对象自己
        """
        ...

    def auto_backup(self,
                    interval: int = None,
                    path: Union[str, Path] = None,
                    new_name: bool = None) -> RecorderSetter:
        """设置自动备份相关参数
        :param interval: 自动保存多少次时触发备份，为0表示不自动备份，为None时不修改已设置值（初始为0）
        :param path: 备份文件存放路径，为None时不修改已设置值（初始为 'backup'）
        :param new_name: 是否用新名称备份，为None时不修改已设置值（初始为True）
        :return: 设置对象自己
        """
        ...

    def table(self, name: Union[str, bool]) -> RecorderSetter:
        """设置默认表名
        :param name: 表名
        :return: 设置对象自己
        """
        ...

    def auto_new_header(self, on_off: bool = True) -> RecorderSetter:
        """数据中有表头不存在的列时是否自动添加到表头，只有xlsx和csv格式有效
        :param on_off: bool表示开关
        :return: 设置对象自己
        """
        ...

    def after(self, data: Any) -> RecorderSetter:
        """设置在数据后面补充的列
        :param data: 列表、元组或字符串，为字符串时则补充一列
        :return: 设置对象自己
        """
        ...

    def before(self, data: Any) -> RecorderSetter:
        """设置在数据前面补充的列
        :param data: 列表、元组或字符串，为字符串时则补充一列
        :return: 设置对象自己
        """
        ...
    # -------------------上级结束-------------------

    def path(self, path: Union[str, Path]) -> RecorderSetter:
        """设置文件路径
        :param path: 文件路径
        :return: 设置对象自己
        """
        ...

    def header(self, header: Union[list, tuple], table: str = None,
               to_file: bool = True, row: int = None) -> RecorderSetter:
        """设置表头。只有 csv 和 xlsx 格式支持设置表头
        :param header: 表头，列表或元组
        :param table: 表名，只xlsx格式文件有效
        :param to_file: 是否写入到文件
        :param row: 指定写入文件的行号，不改变对象已设置的header_row属性，to_file为False时无效
        :return: 设置对象自己
        """
        ...

    def header_row(self, num: int, table=None) -> RecorderSetter:
        """设置标题行号
        :param num: 行号
        :param table: 表名，None为活动表格
        :return: 设置对象自己
        """
        ...

    def encoding(self, encoding: str) -> BaseSetter:
        """设置编码
        :param encoding: 编码格式
        :return: 设置对象自己
        """
        ...

    def delimiter(self, delimiter: str) -> RecorderSetter:
        """设置csv文件分隔符
        :param delimiter: 分隔符
        :return: 设置对象自己
        """
        ...

    def quote_char(self, quote_char: str) -> RecorderSetter:
        """设置csv文件引用符
        :param quote_char: 引用符
        :return: 设置对象自己
        """
        ...

    def follow_styles(self, on_off: bool = True) -> RecorderSetter:
        """设置是否跟随上一行的style，只有xlsx格式有效
        :param on_off: True或False
        :return: 设置对象自己
        """
        ...

    def file_type(self, file_type: str) -> RecorderSetter:
        """指定文件类型，无视文件后缀名
        :param file_type: 后缀名字符串
        :return: 设置对象自己
        """
        ...

    def new_row_height(self, height: float) -> RecorderSetter:
        """设置新行行高，只有xlsx格式有效
        :param height: 行高，传入None清空设置
        :return: 设置对象自己
        """
        ...

    def new_row_styles(self,
                       styles: Union[CellStyle, List[CellStyle], Tuple[CellStyle, ...],
                       Dict[Any, CellStyle], None]) -> RecorderSetter:
        """设置新行样式，只有xlsx格式有效，可传入多个，传入None则取消
        :param styles: 传入CellStyle对象设置整个新行，传入CellStyle对象组成的列表设置多个，传入None清空设置
        :return: 设置对象自己
        """
        ...

    def data_col(self, col: Union[None, str, int]) -> RecorderSetter:
        """设置用于填充数据的列
        :param col: 列号或列名，None表示header后面新列
        :return: 设置对象自己
        """
        ...

    def link_style(self, style: Union[CellStyle, True] = True) -> RecorderSetter:
        """设置单元格的链接样式
        :param style: CellStyle对象
        :return: 设置对象自己
        """
        ...


class DBSetter(BaseSetter):
    _recorder: DBRecorder = ...

    def __init__(self, recorder: DBRecorder): ...

    # -------------------上级开始-------------------
    def cache_size(self, size: int) -> DBSetter:
        """设置缓存大小
        :param size: 缓存大小
        :return: 设置对象自己
        """
        ...

    def show_msg(self, on_off: bool) -> DBSetter:
        """设置是否显示运行信息
        :param on_off: bool表示开关
        :return: 设置对象自己
        """
        ...

    def auto_backup(self,
                    interval: int = None,
                    path: Union[str, Path] = None,
                    new_name: bool = None) -> DBSetter:
        """设置自动备份相关参数
        :param interval: 自动保存多少次时触发备份，为0表示不自动备份，为None时不修改已设置值（初始为0）
        :param path: 备份文件存放路径，为None时不修改已设置值（初始为 'backup'）
        :param new_name: 是否用新名称备份，为None时不修改已设置值（初始为True）
        :return: 设置对象自己
        """
        ...

    def auto_new_header(self, on_off: bool = True) -> DBSetter:
        """数据中有表头不存在的列时是否自动添加到表头，只有xlsx和csv格式有效
        :param on_off: bool表示开关
        :return: 设置对象自己
        """
        ...

    def after(self, data: Any) -> DBSetter:
        """设置在数据后面补充的列
        :param data: 列表、元组或字符串，为字符串时则补充一列
        :return: 设置对象自己
        """
        ...

    def before(self, data: Any) -> DBSetter:
        """设置在数据前面补充的列
        :param data: 列表、元组或字符串，为字符串时则补充一列
        :return: 设置对象自己
        """
        ...
    # -------------------上级结束-------------------

    def path(self, path: Union[str, Path], table: Optional[str] = None) -> DBSetter:
        """重写父类方法
        :param path: 文件路径
        :param table: 数据表名称
        :return: 设置对象自己
        """
        ...

    def table(self, name: Union[str, bool]) -> DBSetter:
        """设置默认表名
        :param name: 表名
        :return: 设置对象自己
        """
        ...


def set_csv_header(recorder: Recorder,
                   header: Header,
                   row: int) -> None:
    """设置csv文件的表头
    :param recorder: Recorder对象
    :param header: 表头列表或元组
    :param row: 行号
    :return: None
    """
    ...


def set_xlsx_header(recorder: Recorder,
                    header: Header,
                    table: str,
                    row: int) -> None:
    """设置xlsx文件的表头
    :param recorder: Recorder对象
    :param header: 表头列表或元组
    :param table: 工作表名称
    :param row: 行号
    :return: None
    """
    ...



# -*- coding:utf-8 -*-
from pathlib import Path
from sqlite3 import connect
from time import sleep

from .base import BaseRecorder
from .setter import DBSetter
from .tools import ok_list_db, is_sigal_data, is_1D_data


class DBRecorder(BaseRecorder):
    _SUPPORTS = ('db',)

    def __init__(self, path=None, cache_size=1000, table=None):
        self._conn = None
        self._cur = None
        super().__init__(None, cache_size)
        if path:
            self.set.path(path, table)
        self._type = 'db'

    @property
    def set(self):
        if self._setter is None:
            self._setter = DBSetter(self)
        return self._setter

    @property
    def tables(self):
        self._connect()
        self._cur.execute("select name from sqlite_master where type='table'")
        tables = self._cur.fetchall()
        self._close_connection()
        return [i[0] for i in tables]

    def add_data(self, data, table=None):
        while self._pause_add:  # 等待其它线程写入结束
            sleep(.02)

        table = table or self.table
        if not isinstance(table, str):
            raise RuntimeError('未指定数据库表名。')

        data = self._handle_data(data)
        self._data.setdefault(table, []).extend(data)

        if 0 < self.cache_size <= self._data_count:
            self.record()

    def run_sql(self, sql, single=True, commit=False):
        self._connect()
        self._cur.execute(sql)
        r = self._cur.fetchone() if single else self._cur.fetchall()
        if commit:
            self._conn.commit()
        self._close_connection()
        return r

    def _connect(self):
        Path(self.path).parent.mkdir(parents=True, exist_ok=True)
        self._conn = connect(self.path)
        self._cur = self._conn.cursor()

    def _close_connection(self):
        if self._conn is not None:
            try:
                self._cur.close()
                self._conn.close()
            except:
                pass

    def _to_database(self, data_list, table, tables):
        if isinstance(data_list[0], dict):  # 检查是否要新增列
            keys = data_list[0].keys()
            for key in keys:
                if str(key) not in tables[table]:
                    if self._auto_new_header:
                        sql = f'ALTER TABLE `{table}` ADD COLUMN `{key}`'
                        self._cur.execute(sql)
                        tables[table].append(key)
                    else:
                        data_list[0].pop(key)

            question_masks = ','.join('?' * len(data_list[0]))
            keys_txt = '`' + '`,`'.join(data_list[0]) + '`'
            values = [ok_list_db(i.values()) for i in data_list]
            sql = f'INSERT INTO `{table}` ({keys_txt}) values ({question_masks})'

        else:
            question_masks = ','.join('?' * len(tables[table]))
            values = data_list
            sql = f'INSERT INTO `{table}` values ({question_masks})'

        self._cur.executemany(sql, values)

    def _record(self):
        self._connect()  # 获取所有表名和列名
        self._cur.execute("select name from sqlite_master where type='table'")
        tables = {}
        for table in self._cur.fetchall():
            self._cur.execute(f"PRAGMA table_info(`{table[0]}`)")
            tables[table[0]] = [i[1] for i in self._cur.fetchall()]

        for table, data in self._data.items():
            data_list = []
            if isinstance(data[0], dict):
                curr_keys = data[0].keys()
            else:
                curr_keys = len(data[0])

            for d in data:
                if isinstance(d, dict):
                    tmp_keys = d.keys()
                    if table not in tables:
                        keys = list(d.keys())
                        self._cur.execute(f"CREATE TABLE `{table}` (`{'`,`'.join(keys)}`)")
                        tables[table] = keys

                else:
                    if table not in tables:
                        self._close_connection()
                        raise TypeError('新建表格首次须接收数据需为dict格式。')
                    tmp_keys = len(d)
                    long = len(tables[table])
                    if long > tmp_keys:
                        d = ok_list_db(d)
                        d.extend([None] * (long - tmp_keys))
                    elif long < tmp_keys:
                        self._close_connection()
                        raise RuntimeError('数据个数大于列数（注意before和after属性）。')

                if tmp_keys != curr_keys:
                    self._to_database(data_list, table, tables)
                    curr_keys = tmp_keys
                    data_list = []

                data_list.append(d)

            if data_list:
                self._to_database(data_list, table, tables)

        self._conn.commit()
        self._close_connection()

    def _handle_data(self, data):
        if is_sigal_data(data):
            data = (self._make_final_data(self, (data,)),)
            self._data_count += 1
        elif not data:
            data = (self._make_final_data(self, tuple()),)
            self._data_count += 1
        elif is_1D_data(data):
            data = [self._make_final_data(self, data)]
            self._data_count += 1
        else:  # 二维数组
            data = [self._make_final_data(self, (d,)) if is_sigal_data(d)
                    else self._make_final_data(self, d) for d in data]
            self._data_count += len(data)
        return data



# -*- coding:utf-8 -*-
from threading import Lock
from typing import Literal, Optional, Any, Union

from openpyxl.cell import Cell
from openpyxl.styles import Alignment, Font, Border, Fill, Protection, Side, PatternFill, Color

LINES = Literal['dashDot', 'dashDotDot', 'dashed', 'dotted', 'double', 'hair', 'medium', 'mediumDashDot',
'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin', None]


class CellStyle(object):
    font_args: tuple = ...
    border_args: tuple = ...
    alignment_args: tuple = ...
    protection_args: tuple = ...
    gradient_fill_args: tuple = ...
    pattern_fill_args: tuple = ...

    def __init__(self) -> None:
        """用于管理单元格样式的类"""
        self._font: Optional[CellFont] = ...
        self._border: Optional[CellBorder] = ...
        self._pattern_fill: Optional[CellPatternFill] = ...
        self._gradient_fill: Optional[CellGradientFill] = ...
        self._number_format: Optional[CellNumberFormat] = ...
        self._protection: Optional[CellProtection] = ...
        self._alignment: Optional[CellAlignment] = ...
        self._Font: Optional[Font] = ...
        self._Border: Optional[Border] = ...
        self._Alignment: Optional[Alignment] = ...
        self._Fill: Optional[Fill] = ...
        self._Protection: Optional[Protection] = ...
        self.height: Optional[float] = ...
        self.width: Optional[float] = ...

    @property
    def font(self) -> CellFont:
        """返回用于设置单元格字体的对象"""
        ...

    @property
    def border(self) -> CellBorder:
        """返回用于设置单元格边框的对象"""
        ...

    @property
    def alignment(self) -> CellAlignment:
        """返回用于设置单元格对齐选项的对象"""
        ...

    @property
    def pattern_fill(self) -> CellPatternFill:
        """返回用于设置单元格图案填充的对象"""
        ...

    @property
    def gradient_fill(self) -> CellGradientFill:
        """返回用于设置单元格渐变填充的对象"""
        ...

    @property
    def number_format(self) -> CellNumberFormat:
        """返回用于设置单元格数字格式的对象"""
        ...

    @property
    def protection(self) -> CellProtection:
        """返回用于设置单元格保护选项的对象"""
        ...

    def to_cell(self, cell: Cell, replace: bool = True) -> None:
        """把当前样式复制到目标单元格
        :param cell: 被设置样式的单元格对象
        :param replace: 是否直接替换目标单元格的样式，是的话效率较高，但不能保留未被设置的原有样式项
        :return: None
        """
        ...

    def set_size(self, width: float = None, height: float = None) -> CellStyle:
        """设置单元格宽和高，width和height都是None时清除宽高设置
        :param width: 单元格宽度，为None则不改变
        :param height: 单元格高度，为None则不改变
        :return: None
        """
        ...

    def set_bgColor(self, color: Union[None, str, tuple, Color]) -> CellStyle:
        """设置背景颜色
        :param color: 格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: 样式对象本身
        """
        ...

    def set_txtSize(self, size: float = None, bold: bool = None) -> CellStyle:
        """设置字体大小和是否加粗
        :param size: 大小，为None时不修改
        :param bold: 是否加粗，为None时不修改
        :return: 样式对象本身
        """
        ...

    def set_txtColor(self, color: Union[None, str, tuple, Color]) -> CellStyle:
        """设置文本颜色
        :param color: 字体颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: 样式对象本身
        """
        ...

    def set_bold(self, on_off: Optional[bool] = True) -> CellStyle:
        """设置字体是否加粗
        :param on_off: bool表示有或无
        :return: 样式对象本身
        """
        ...

    def set_delLine(self, on_off: Optional[bool] = True) -> CellStyle:
        """设置是否有删除线
        :param on_off: bool表示有或无
        :return: 样式对象本身
        """
        ...

    def set_underLine(self, on_off: Optional[bool] = True) -> CellStyle:
        """设置是否有下划线线
        :param on_off: bool表示有或无
        :return: 样式对象本身
        """
        ...

    def _cover_to_cell(self, cell: Cell) -> None:
        """把当前样式复制到目标单元格，只覆盖有设置的项，没有设置的原有的项不变
        :param cell: 被设置样式的单元格对象
        :return: None
        """
        ...

    def _replace_to_cell(self, cell: Cell) -> None:
        """把当前样式复制到目标单元格，覆盖原有的设置
        :param cell: 被设置样式的单元格对象
        :return: None
        """
        ...


def _handle_args(args: tuple, src: Any, target: Any) -> dict: ...


class CellFont(object):
    _LINE_STYLES: tuple = ...
    _SCHEMES: tuple = ...
    _VERT_ALIGNS: tuple = ...

    def __init__(self):
        self.name: str = ...
        self.charset: int = ...
        self.size: float = ...
        self.bold: bool = ...
        self.italic: bool = ...
        self.strike: bool = ...
        self.outline: bool = ...
        self.shadow: bool = ...
        self.condense: bool = ...
        self.extend: bool = ...
        self.underline: Literal['single', 'double', 'singleAccounting', 'doubleAccounting'] = ...
        self.vertAlign: Literal['superscript', 'subscript', 'baseline'] = ...
        self.color: Union[Color, str] = ...
        self.scheme: Literal['major', 'minor'] = ...

    def set_name(self, name: Optional[str]) -> None:
        """设置字体
        :param name: 字体名称，None表示恢复默认
        :return: None
        """
        ...

    def set_charset(self, charset: Optional[int]) -> None:
        """设置编码
        :param charset: 字体编码，int格式，None表示恢复默认
        :return: None
        """
        ...

    def set_size(self, size: Optional[float]) -> None:
        """设置字体大小
        :param size: 字体大小，None表示恢复默认
        :return: None
        """
        ...

    def set_bold(self, on_off: Optional[bool]) -> None:
        """设置是否加粗
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_italic(self, on_off: Optional[bool]) -> None:
        """设置是否斜体
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_strike(self, on_off: Optional[bool]) -> None:
        """设置是否有删除线
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_outline(self, on_off: Optional[bool]) -> None:
        """设置outline
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_shadow(self, on_off: Optional[bool]) -> None:
        """设置是否有阴影
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_condense(self, on_off: Optional[bool]) -> None:
        """设置condense
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_extend(self, on_off: Optional[bool]) -> None:
        """设置extend
        :param on_off: bool表示开关，None表示恢复默认
        :return: None
        """
        ...

    def set_color(self, color: Union[None, str, tuple, Color]) -> None:
        """设置字体颜色
        :param color: 字体颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_underline(self,
                      option: Literal['single', 'double', 'singleAccounting', 'doubleAccounting', None]) -> None:
        """设置下划线
        :param option: 下划线类型，可选 'single', 'double', 'singleAccounting', 'doubleAccounting'，None表示恢复默认
        :return: None
        """
        ...

    def set_vertAlign(self, option: Literal['superscript', 'subscript', 'baseline', None]) -> None:
        """设置上下标
        :param option: 可选 'superscript', 'subscript', 'baseline'，None表示恢复默认
        :return: None
        """
        ...

    def set_scheme(self, option: Literal['major', 'minor', None]) -> None:
        """设置scheme
        :param option: 可选 'major', 'minor'，None表示恢复默认
        :return: None
        """
        ...


class CellBorder(object):
    _LINE_STYLES: tuple = ...

    def __init__(self):
        self.start: Side = ...
        self.end: Side = ...
        self.left: Side = ...
        self.right: Side = ...
        self.top: Side = ...
        self.bottom: Side = ...
        self.diagonal: Side = ...
        self.vertical: Side = ...
        self.horizontal: Side = ...
        self.horizontal: Side = ...
        self.outline: bool = ...
        self.diagonalUp: bool = ...
        self.diagonalDown: bool = ...

    def set_start(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置start
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_end(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置end
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_left(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置左边框
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_right(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置右边框
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_top(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置上边框
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_bottom(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置下边框
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_quad(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置四个边框边框
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_diagonal(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置对角线
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_vertical(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置垂直中线
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_horizontal(self, style: LINES, color: Union[None, str, tuple, Color]) -> None:
        """设置水平中线
        :param style: 线形，'dashDot','dashDotDot', 'dashed','dotted', 'double','hair', 'medium', 'mediumDashDot',
                      'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin'，None表示恢复默认
        :param color: 边框颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_outline(self, on_off: bool) -> None:
        """
        :param on_off: bool表示开关
        :return: None
        """
        ...

    def set_diagonalDown(self, on_off: bool) -> None:
        """
        :param on_off: bool表示开关
        :return: None
        """
        ...

    def set_diagonalUp(self, on_off: bool) -> None:
        """
        :param on_off: bool表示开关
        :return: None
        """
        ...


class CellAlignment(object):
    _horizontal_alignments: tuple = ...
    _vertical_alignments: tuple = ...

    def __init__(self):
        self.horizontal = 'notSet'
        self.vertical = 'notSet'
        self.indent = 'notSet'
        self.relativeIndent = 'notSet'
        self.justifyLastLine = 'notSet'
        self.readingOrder = 'notSet'
        self.textRotation = 'notSet'
        self.wrapText = 'notSet'
        self.shrinkToFit = 'notSet'

    def set_horizontal(self,
                       horizontal: Literal['general', 'left', 'center', 'right', 'fill', 'justify', 'centerContinuous',
                       'distributed', None]) -> None:
        """设置水平位置
        :param horizontal: 可选：'general', 'left', 'center', 'right', 'fill', 'justify', 'centerContinuous',
                                'distributed'，None表示恢复默认
        :return: None
        """
        ...

    def set_vertical(self, vertical: Literal['top', 'center', 'bottom', 'justify', 'distributed', None]) -> None:
        """设置垂直位置
        :param vertical: 可选：'top', 'center', 'bottom', 'justify', 'distributed'，None表示恢复默认
        :return: None
        """
        ...

    def set_indent(self, indent: int) -> None:
        """设置缩进
        :param indent: 缩进数值，0到255
        :return: None
        """
        ...

    def set_relativeIndent(self, indent: int) -> None:
        """设置相对缩进
        :param indent: 缩进数值，-255到255
        :return: None
        """
        ...

    def set_justifyLastLine(self, on_off: Optional[bool]) -> None:
        """设置justifyLastLine
        :param on_off: bool表示开或关，None表示恢复默认
        :return: None
        """
        ...

    def set_readingOrder(self, value: int) -> None:
        """设置readingOrder
        :param value: 不小于0的数字
        :return: None
        """
        ...

    def set_textRotation(self, value: int) -> None:
        """设置文本旋转角度
        :param value: 0-180或255
        :return: None
        """
        ...

    def set_wrapText(self, on_off: Optional[bool]) -> None:
        """设置wrapText
        :param on_off: bool表示开或关，None表示恢复默认
        :return: None
        """
        ...

    def set_shrinkToFit(self, on_off: Optional[bool]) -> None:
        """设置shrinkToFit
        :param on_off: bool表示开或关，None表示恢复默认
        :return: None
        """
        ...


class CellGradientFill(object):
    def __init__(self):
        self.type: str = ...
        self.degree: float = ...
        self.left: float = ...
        self.right: float = ...
        self.top: float = ...
        self.bottom: float = ...
        self.stop: Union[list, tuple] = ...

    def set_type(self, name: Literal['linear', 'path']) -> None:
        """设置类型
        :param name: 可选：'linear', 'path'
        :return: None
        """
        ...

    def set_degree(self, value: float) -> None:
        """设置程度
        :param value: 数值
        :return: None
        """
        ...

    def set_left(self, value: float) -> None:
        """设置left
        :param value: 数值
        :return: None
        """
        ...

    def set_right(self, value: float) -> None:
        """设置right
        :param value: 数值
        :return: None
        """
        ...

    def set_top(self, value: float) -> None:
        """设置top
        :param value: 数值
        :return: None
        """
        ...

    def set_bottom(self, value: float) -> None:
        """设置bottom
        :param value: 数值
        :return: None
        """
        ...

    def set_stop(self, values: Union[list, tuple]) -> None:
        """设置stop
        :param values: 数值
        :return: None
        """
        ...


class CellPatternFill(object):
    _FILES: tuple = ...

    def __init__(self):
        self.patternType: str = ...
        self.fgColor: Union[str, Color] = ...
        self.bgColor: Union[str, Color] = ...

    def set_patternType(self, name: Literal[
        'none', 'solid', 'darkDown', 'darkGray', 'darkGrid', 'darkHorizontal', 'darkTrellis', 'darkUp',
        'darkVertical', 'gray0625', 'gray125', 'lightDown', 'lightGray', 'lightGrid', 'lightHorizontal',
        'lightTrellis', 'lightUp', 'lightVertical', 'mediumGray', None]) -> None:
        """设置类型
        :param name: 可选：'none', 'solid', 'darkDown', 'darkGray', 'darkGrid', 'darkHorizontal', 'darkTrellis',
                          'darkUp', 'darkVertical', 'gray0625', 'gray125', 'lightDown', 'lightGray', 'lightGrid',
                          'lightHorizontal', 'lightTrellis', 'lightUp', 'lightVertical', 'mediumGray'，None为恢复默认
        :return: None
        """
        ...

    def set_fgColor(self, color: Union[None, str, tuple, Color]) -> None:
        """设置前景色
        :param color: 颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...

    def set_bgColor(self, color: Union[None, str, tuple, Color]) -> None:
        """设置背景色
        :param color: 颜色，格式：'FFFFFF', '255,255,255', (255, 255, 255), Color对象均可，None表示恢复默认
        :return: None
        """
        ...


class CellNumberFormat(object):
    def __init__(self):
        self.format: str = 'notSet'

    def set_format(self, string: Optional[str]) -> None:
        """设置数字格式
        :param string: 格式字符串，为None时恢复默认，格式很多具体在`openpyxl.numbers`查看
        :return: None
        """
        ...


class CellProtection(object):
    def __init__(self):
        self.hidden: bool = ...
        self.locked: bool = ...

    def set_hidden(self, on_off: bool) -> None:
        """设置是否隐藏
        :param on_off: bool表示开关
        :return: None
        """
        ...

    def set_locked(self, on_off: bool) -> None:
        """设置是否锁定
        :param on_off: bool表示开关
        :return: None
        """
        ...


class CellStyleCopier(object):
    def __init__(self, from_cell: Cell):
        self._style = ...
        self._font: Font = ...
        self._border: Border = ...
        self._fill: Fill = ...
        self._number_format = ...
        self._protection: Protection = ...
        self._alignment: Alignment = ...

    def to_cell(self, cell: Cell) -> None:
        """把当前样式复制到目标单元格
        :param cell: 被设置样式的单元格对象
        :return: None
        """
        ...


def get_color_code(color: Union[str, tuple, Color]) -> str:
    """将颜色拼音转为代码
    :param color: 颜色名称或代码字符串
    :return: 颜色代码
    """
    ...


class NoneStyle(object):
    _instance_lock: Lock = ...

    def __init__(self):
        self._font: Font = ...
        self._border: Border = ...
        self._alignment: Alignment = ...
        self._fill: PatternFill = ...
        self._number_format: str = ...
        self._protection: Protection = ...

    def __new__(cls, *args, **kwargs): ...

    def to_cell(self, cell: Cell, replace: bool = True) -> None:
        """把当前样式复制到目标单元格
        :param cell: 被设置样式的单元格对象
        :param replace: 用于对齐
        :return: None
        """
        ...



# -*- coding:utf-8 -*-
"""
_header格式：{表名: Header对象}
表名为None表示默认sheet或csv文件
未设置header时值为None
"""
from csv import writer as csv_writer
from io import TextIOWrapper
from pathlib import Path
from typing import Any, Optional, Union, List, Dict, Tuple, Callable, Iterable

from openpyxl.worksheet.worksheet import Worksheet

from .base import BaseRecorder
from .setter import RecorderSetter
from .cell_style import CellStyle
from .tools import Header, RowData


class Recorder(BaseRecorder):
    _set: RecorderSetter = ...
    _row_height: Optional[float] = ...
    _follow_styles: bool = ...
    _styles: Union[CellStyle, List[CellStyle], Tuple[CellStyle], Dict[str, CellStyle], None] = ...
    _quote_char: str = ...
    _delimiter: str = ...
    _header: Dict[Optional[str], Optional[Header]] = ...
    _header_row: Dict[Optional[str], int] = ...
    _fast: bool = ...
    _methods: dict = ...
    _link_style: Optional[CellStyle] = ...
    _data: Dict[Optional[str], list] = ...
    data: Dict[Optional[str], list] = ...
    data_col: int = ...

    def __init__(self, path: Union[str, Path] = None, cache_size: int = 1000):
        """用于缓存并记录数据，可在达到一定数量时自动记录，以降低文件读写次数，减少开销
        :param path: 保存的文件路径
        :param cache_size: 每接收多少条记录写入文件，0为不自动写入
        """
        ...

    def _set_methods(self, file_type: str) -> None:
        """设置各种情况下使用的方法"""
        ...

    @property
    def set(self) -> RecorderSetter:
        """返回用于设置属性的对象"""
        ...

    @property
    def delimiter(self) -> str:
        """返回csv文件分隔符"""
        ...

    @property
    def quote_char(self) -> str:
        """返回csv文件引用符"""
        ...

    @property
    def header(self) -> Header:
        """返回表头，只支持csv和xlsx格式"""
        ...

    def add_data(self, data: Any,
                 coord: Union[list, Tuple[Union[None, int, str], Union[None, int, str]], str, int] = None,
                 table: Union[str, bool] = None) -> None:
        """添加数据，可一次添加多条数据
        :param data: 插入的数据，任意格式，可以为二维数据
        :param coord: 要添加数据的坐标，可输入行号、列号或行列坐标，当格式不是xlsx或csv时无效，eg.'a3'、1、[3, 1]、'c'、'-3'
        :param table: 要写入的数据表，仅支持xlsx格式。为None表示用set.table()方法设置的值，为True表示活动的表格
        :return: None
        """
        ...

    def set_link(self,
                 coord: Union[int, str, tuple, list],
                 link: Union[str, dict],
                 content: Union[None, int, str, float] = None,
                 table: Union[str, bool] = None) -> None:
        """为单元格设置超链接
        :param coord: 单元格坐标
        :param link: 超链接，为None时删除链接
        :param content: 单元格内容
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def set_img(self,
                coord: Union[int, str, tuple, list],
                img_path: Union[None, str, Path, dict],
                width: float = None,
                height: float = None,
                table: Union[str, bool] = None) -> None:
        """向单元格设置图片
        :param coord: 单元格坐标
        :param img_path: 图片路径
        :param width: 图片宽
        :param height: 图片高
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def set_styles(self,
                   coord: Union[int, str, tuple, list],
                   styles: Union[CellStyle, dict, list, tuple],
                   replace: bool = True,
                   table: Union[str, bool] = None) -> None:
        """为单元格设置样式，可批量设置范围内的单元格
        :param coord: 单元格坐标，输入数字可设置整行，输入列名字符串可设置整列，输入'A1:C5'、'a:d'、'1:5'格式可设置指定范围
        :param styles: CellStyle对象，为None则清除单元格样式
        :param replace: 是否直接替换已有样式，运行效率较高，但不能单独修改某个属性
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def set_row_height(self, row: Union[int, str], height: float,
                       table: Union[str, bool] = None) -> None:
        """设置行高，可设置连续多行
        :param row: 行号，可传入范围，如'1:4'
        :param height: 行高
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def set_col_width(self, col: Union[int, str], width: float,
                      table: Union[str, bool] = None) -> None:
        """设置列宽，可设置连续多列
        :param col: 列号，数字或字母，可传入范围，如'1:4'、'a:d'
        :param width: 列宽
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def _set_link(self,
                  coord: Union[int, str, tuple, list],
                  link: Union[str, dict],
                  content: Union[None, int, str, float] = None,
                  table: Union[str, bool] = None) -> None:
        """为单元格设置超链接
        :param coord: 单元格坐标
        :param link: 超链接，为None时删除链接
        :param content: 单元格内容
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def _set_img(self,
                 coord: Union[int, str, tuple, list],
                 img_path: Union[None, str, Path, dict],
                 width: float = None,
                 height: float = None,
                 table: Union[str, bool] = None) -> None:
        """向单元格设置图片
        :param coord: 单元格坐标
        :param img_path: 图片路径
        :param width: 图片宽
        :param height: 图片高
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def _set_styles(self,
                    coord: Union[int, str, tuple, list],
                    style: Union[CellStyle, dict],
                    replace: bool = True,
                    table: Union[str, bool] = None) -> None:
        """为单元格设置样式，可批量设置范围内的单元格
        :param coord: 单元格坐标，输入数字可设置整行，输入列名字符串可设置整列，输入'A1:C5'、'a:d'、'1:5'格式可设置指定范围
        :param style: CellStyle对象，为None则清除单元格样式
        :param replace: 是否直接替换已有样式，运行效率较高，但不能单独修改某个属性
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def _set_row_height(self, row: Union[int, str], height: float,
                        table: Union[str, bool] = None) -> None:
        """设置行高，可设置连续多行
        :param row: 行号，可传入范围，如'1:4'
        :param height: 行高
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def _set_col_width(self, col: Union[int, str], width: float,
                       table: Union[str, bool] = None) -> None:
        """设置列宽，可设置连续多列
        :param col: 列号，数字或字母，可传入范围，如'1:4'、'a:d'
        :param width: 列宽
        :param table: 数据表名，仅支持xlsx格式。为None表示用set.table()方法设置的值，为bool表示活动的表格
        :return: None
        """
        ...

    def _add(self, data: dict, table: Optional[str], to_slow: bool, num: int, add_method: callable) -> None:
        """为单元格设置样式，可批量设置范围内的单元格
        :param data: 数据，[dict, ...]格式
        :param table: 表格名称，None为活动表格
        :param to_slow: 是否转到slow模式
        :param num: 增加的数据量
        :return: None
        """
        ...

    def _add_data_any(self, data: dict, table: Optional[str]) -> None:
        """添加data到_data的操作，适用于任意类型文件
        :param data: 数据
        :param table: 要添加数据的表
        :return: None
        """
        ...

    def _add_data_txt(self, data: dict, table: Optional[str]) -> None:
        """添加data到_data的操作，适用于除xlsx以外的文件类型
        :param data: 数据
        :param table: 要添加数据的表
        :return: None
        """
        ...

    def _add_others(self, data: dict, table: Optional[str]) -> None:
        """添加style、link等到_data的操作
        :param data: 数据
        :param table: 要添加数据的表
        :return: None
        """
        ...

    def rows(self,
             key_cols: Union[str, int, list, tuple, bool] = True,
             sign_col: Union[str, int, True] = True,
             is_header: bool = False,
             signs: Any = None,
             deny_sign: bool = False,
             count: int = None,
             begin_row: Optional[int] = None) -> List[RowData]:
        """返回符合条件的行数据，可指定只要某些列
        :param key_cols: 作为关键字的列，可以是多列，为True获取所有列
        :param sign_col: 用于筛选数据的列，为True获取所有行
        :param is_header: key_cols和sign_col是str时，表示header值还是列名
        :param signs: 按这个值筛选目标行，可用list, tuple, set设置多个
        :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
        :param count: 获取多少条数据，为None获取所有
        :param begin_row: 数据开始的行，None表示header_row后面一行
        :return: RowData对象
        """
        ...

    def _handle_data(self, data: Any, coord: tuple) -> Tuple[dict, int]:
        """把数据处理成存储格式
        :param data: 要处理的数据
        :param coord: 单元格坐标
        :return: (处理后的数据, 数据长度)
        """
        ...

    def _record(self) -> None:
        """记录数据"""
        ...

    def _fast_mode(self) -> None:
        """切换到fast模式"""
        ...

    def _slow_mode(self) -> None:
        """切换到slow模式"""
        ...

    def _to_xlsx_fast(self) -> None:
        """fast模式填写数据到xlsx文件"""
        ...

    def _to_csv_fast(self) -> None:
        """fast模式填写数据到csv文件"""
        ...

    def _to_csv_slow(self) -> None:
        """slow模式填写数据到csv文件"""
        ...

    def _to_txt_fast(self) -> None:
        """记录数据到txt文件"""
        ...

    def _to_jsonl_fast(self) -> None:
        """记录数据到jsonl文件"""
        ...

    def _to_json_fast(self) -> None:
        """记录数据到json文件"""
        ...

    def _to_txt_slow(self) -> None:
        """记录数据到txt文件"""
        ...

    def _to_jsonl_slow(self) -> None:
        """记录数据到jsonl文件"""
        ...

    def _to_json_slow(self) -> None:
        """记录数据到json文件"""
        ...


def handle_txt_lines(data_lst: list, lines: list, val: Any, method: Callable) -> None:
    """txt、json、jsonl格式相同的写入逻辑
    :param data_lst: 数据总列表
    :param lines: readlines()从文件读取的原数据列表
    :param val: 插入空行时的值
    :param method: 处理单个数据使用的方法
    :return: None
    """
    ...


def handle_txt_data(lines: list, num: int, data: Union[dict, list]) -> None:
    """处理txt格式单个数据的方法，对应handle_txt_lines()的method
    :param lines: list格式的文件数据
    :param num: 行号
    :param data: 要写入的数据
    :return: None
    """
    ...


def handle_jsonl_data(lines: list, num: int, data: Union[dict, list]) -> None:
    """处理jsonl格式单个数据的方法，对应handle_txt_lines()的method
    :param lines: list格式的文件数据
    :param num: 行号
    :param data: 要写入的数据
    :return: None
    """
    ...


def handle_json_data(lines: list, num: int, data: Union[dict, list]) -> None:
    """处理json格式单个数据的方法，对应handle_txt_lines()的method
    :param lines: list格式的文件数据
    :param num: 行号
    :param data: 要写入的数据
    :return: None
    """
    ...


def get_header(recorder: Recorder, ws: Worksheet = None) -> Header:
    """获取当前指定的table的header
    :param recorder: Recorder对象
    :param ws: Worksheet对象
    :return: Header对象
    """
    ...


def handle_new_sheet(recorder: Recorder, ws: Worksheet, data: list) -> int:
    """从设置或第一条dict数据获取表头并向新表写入
    :param recorder: Recorder对象
    :param ws: 数据表对象
    :param data: 对应数据表的数据列表
    :return: 开始写数据的行的前一行
    """
    ...


def get_first_dict(data: list) -> dict:
    """判断数据集第一条是否dict，如果第一条是二维数据，判断其第一条是否dict，是则返回它
    :param data: 数据列表
    :return: 第一条dict格式数据
    """
    ...


def get_xlsx_rows(recorder: Recorder,
                  header: Header, key_cols: Union[list, True],
                  begin_row: Optional[int], sign_col: Union[str, int, bool], sign: Any,
                  deny_sign: bool, count: int, ws: Worksheet) -> List[RowData]:
    """获取xlsx文件指定行数据
    :param recorder: Recorder对象
    :param header: Header对象
    :param key_cols: 要获取的列，为True获取所有，可指定多列
    :param begin_row: 开始行号
    :param sign_col: 作为条件的列
    :param sign: 按这个值筛选目标行，可设置多个
    :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
    :param count: 获取多少条数据，为None获取所有
    :param ws: Worksheet对象
    :return: 获取到的数据列表
    """
    ...


def get_csv_rows(recorder: Recorder,
                 header: Header, key_cols: Union[list, True],
                 begin_row: Optional[int], sign_col: Union[str, int, bool], sign: Any,
                 deny_sign: bool, count: int, ws: Worksheet) -> List[RowData]:
    """获取xlsx文件指定行数据
    :param recorder: Recorder对象
    :param header: Header对象
    :param key_cols: 要获取的列，为True获取所有，可指定多列
    :param begin_row: 开始行号
    :param sign_col: 作为条件的列
    :param sign: 按这个值筛选目标行，可设置多个
    :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
    :param count: 获取多少条数据，为None获取所有
    :param ws: 与get_xlsx_rows()对应
    :return: 获取到的数据列表
    """
    ...


def get_xlsx_rows_with_count(key_cols: Union[list, True], deny_sign: bool, header: Header, rows: Iterable,
                             begin_row: Optional[int], sign_col: Union[str, int, bool], sign: Any,
                             count: int) -> List[RowData]:
    """执行从xlsx中获取数据，有指定数量
    :param key_cols: 要获取的列，True为所有
    :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
    :param header: Header对象
    :param rows: 行组成的列表
    :param begin_row: 开始行号
    :param sign_col: 用于筛选数据的列
    :param sign: 用于筛选数据的值
    :param count: 数据总条数
    :return: 数据对象列表
    """
    ...


def get_xlsx_rows_without_count(key_cols: Union[list, True], deny_sign: bool, header: Header, rows: Iterable,
                                begin_row: Optional[int], sign_col: Union[str, int, bool], sign: Any) -> List[RowData]:
    """执行从xlsx中获取全部数据
    :param key_cols: 要获取的列，True为所有
    :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
    :param header: Header对象
    :param rows: 行组成的列表
    :param begin_row: 开始行号
    :param sign_col: 用于筛选数据的列
    :param sign: 用于筛选数据的值
    :return: 数据对象列表
    """
    ...


def get_csv_rows_with_count(lines, begin_row: Optional[int], sign_col, sign, deny_sign: bool,
                            key_cols: Union[list, True], res, header: Header, count: int) -> List[RowData]:
    """执行从csv中获取数据，有指定数量
    :param lines:
    :param begin_row: 开始行号
    :param sign_col: 用于筛选数据的列
    :param sign: 用于筛选数据的值
    :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
    :param key_cols: 要获取的列，True为所有
    :param res: 结果列表
    :param header: Header对象
    :param count: 数据总条数
    :return: 数据对象列表
    """
    ...


def get_csv_rows_without_count(lines, begin_row: Optional[int], sign_col: Union[str, int, bool], sign: Any,
                               deny_sign: bool, key_cols: Union[list, True], res, header: Header) -> List[RowData]:
    """执行从csv中获取全部数据
    :param lines:
    :param begin_row: 开始行号
    :param sign_col: 用于筛选数据的列
    :param sign: 用于筛选数据的值
    :param deny_sign: 是否反向匹配sign，即筛选指不是sign的行
    :param key_cols: 要获取的列，True为所有
    :param res: 结果列表
    :param header: Header对象
    :return: 数据对象列表
    """
    ...


def get_and_set_csv_header(recorder: Recorder, new_csv: bool, file: TextIOWrapper, writer: csv_writer) -> None:
    """从csv获取表头或把已获取的表头设置到新csv
    :param recorder: Recorder对象
    :param new_csv: 是否新csv文件
    :param file: 文件对象
    :param writer: csv writer对象
    :return: None
    """
    ...


def link2ws(**kwargs) -> None:
    """把擦后入到单元格"""
    ...


def img2ws(**kwargs) -> None:
    """把图片到单元格"""
    ...


def width2ws(**kwargs) -> None:
    """把列宽设置到数据表"""
    ...


def height2ws(**kwargs) -> None:
    """把行高设置到数据表"""
    ...



# -*- coding:utf-8 -*-
from copy import copy
from threading import Lock

from openpyxl.styles import Alignment, Font, Side, Border, Protection, GradientFill, PatternFill, Color
from openpyxl.utils import get_column_letter


class CellStyle(object):
    font_args = ('name', 'size', 'charset', 'underline', 'color', 'scheme', 'vertAlign',
                 'bold', 'italic', 'strike', 'outline', 'shadow', 'condense', 'extend')
    border_args = ('start', 'end', 'left', 'right', 'top', 'bottom', 'diagonal', 'vertical', 'horizontal',
                   'horizontal', 'outline', 'diagonalUp', 'diagonalDown')
    alignment_args = ('horizontal', 'vertical', 'indent', 'relativeIndent', 'justifyLastLine', 'readingOrder',
                      'textRotation', 'wrapText', 'shrinkToFit')
    protection_args = ('locked', 'hidden')
    gradient_fill_args = ('type', 'degree', 'left', 'right', 'top', 'bottom', 'stop')
    pattern_fill_args = ('patternType', 'fgColor', 'bgColor')

    def __init__(self):
        self._font = None
        self._border = None
        self._alignment = None
        self._pattern_fill = None
        self._gradient_fill = None
        self._number_format = None
        self._protection = None

        # 用于覆盖目标单元格的对象
        self._Font = None
        self._Border = None
        self._Alignment = None
        self._Fill = None
        self._Protection = None

        self.height = None
        self.width = None

    @property
    def font(self):
        if self._font is None:
            self._font = CellFont()
        return self._font

    @property
    def border(self):
        if self._border is None:
            self._border = CellBorder()
        return self._border

    @property
    def alignment(self):
        if self._alignment is None:
            self._alignment = CellAlignment()
        return self._alignment

    @property
    def pattern_fill(self):
        self._gradient_fill = None
        if self._pattern_fill is None:
            self._pattern_fill = CellPatternFill()
        return self._pattern_fill

    @property
    def gradient_fill(self):
        self._pattern_fill = None
        if self._gradient_fill is None:
            self._gradient_fill = CellGradientFill()
        return self._gradient_fill

    @property
    def number_format(self):
        if self._number_format is None:
            self._number_format = CellNumberFormat()
        return self._number_format

    @property
    def protection(self):
        if self._protection is None:
            self._protection = CellProtection()
        return self._protection

    def to_cell(self, cell, replace=True):
        if replace:
            self._replace_to_cell(cell)
        else:
            self._cover_to_cell(cell)
        if self.height is not None:
            cell.parent.row_dimensions[cell.row].height = self.height
        if self.width is not None:
            cell.parent.column_dimensions[get_column_letter(cell.column)].width = self.width

    def set_size(self, width=None, height=None):
        if width == height is None:
            self.width = self.height = None
            return self
        if width is not None:
            self.width = width
        if height is not None:
            self.height = height
        return self

    def set_bgColor(self, color):
        self.pattern_fill.set_fgColor(color)
        return self

    def set_txtColor(self, color):
        self.font.set_color(color)
        return self

    def set_txtSize(self, size=None, bold=None):
        if size is not None:
            self.font.set_size(size)
        if bold is not None:
            self.font.set_bold(bold)
        return self

    def set_bold(self, on_off=True):
        self.font.set_bold(on_off)
        return self

    def set_delLine(self, on_off=True):
        self.font.set_strike(on_off)
        return self

    def set_underLine(self, on_off=True):
        self.font.set_underline('single' if on_off else None)
        return self

    def _cover_to_cell(self, cell):
        if self._font:
            d = _handle_args(self.font_args, self._font, cell.font)
            d['family'] = cell.font.family
            cell.font = Font(**d)

        if self._border:
            d = _handle_args(self.border_args, self._border, cell.border)
            cell.border = Border(**d)

        if self._alignment:
            d = _handle_args(self.alignment_args, self._alignment, cell.alignment)
            cell.alignment = Alignment(**d)

        if self._pattern_fill:
            f = None if 'fills.GradientFill' in str(cell.fill) else cell.fill
            d = _handle_args(self.pattern_fill_args, self._pattern_fill, f)
            cell.fill = PatternFill(**d)

        elif self._gradient_fill:
            f = None if 'fills.PatternFill' in str(cell.fill) else cell.fill
            d = _handle_args(self.gradient_fill_args, self._gradient_fill, f)
            cell.fill = GradientFill(**d)

        if self._number_format and self._number_format.format != 'notSet':
            cell.number_format = self._number_format.format

        if self._protection:
            d = _handle_args(self.protection_args, self._protection, cell.protection)
            cell.protection = Protection(**d)

    def _replace_to_cell(self, cell):
        if self._font:
            if self._Font is None:
                d = _handle_args(self.font_args, self._font, None)
                self._Font = Font(**d)
            cell.font = self._Font

        if self._border:
            if self._Border is None:
                d = _handle_args(self.border_args, self._border, None)
                self._Border = Border(**d)
            cell.border = self._Border

        if self._alignment:
            if self._Alignment is None:
                d = _handle_args(self.alignment_args, self._alignment, None)
                self._Alignment = Alignment(**d)
            cell.alignment = self._Alignment

        if self._pattern_fill:
            if not isinstance(self._Fill, PatternFill):
                d = _handle_args(self.pattern_fill_args, self._pattern_fill, None)
                self._Fill = PatternFill(**d)
            cell.fill = self._Fill

        elif self._gradient_fill:
            if not isinstance(self._Fill, GradientFill):
                d = _handle_args(self.gradient_fill_args, self._gradient_fill, None)
                self._Fill = GradientFill(**d)
            cell.fill = self._Fill

        if self._number_format and self._number_format.format != 'notSet':
            cell.number_format = self._number_format.format

        if self._protection:
            if self._Protection is None:
                d = _handle_args(self.protection_args, self._protection, None)
                self._Protection = Protection(**d)
            cell.protection = self._Protection


def _handle_args(args, src, target=None):
    d = {}
    for arg in args:
        tmp = getattr(src, arg)
        if tmp != 'notSet':
            d[arg] = tmp
        elif target:
            d[arg] = getattr(target, arg)
    return d


class CellFont(object):
    _LINE_STYLES = ('single', 'double', 'singleAccounting', 'doubleAccounting', None)
    _SCHEMES = ('major', 'minor', None)
    _VERT_ALIGNS = ('superscript', 'subscript', 'baseline', None)

    def __init__(self):
        self.name = 'notSet'
        self.charset = 'notSet'
        self.size = 'notSet'
        self.bold = 'notSet'
        self.italic = 'notSet'
        self.strike = 'notSet'
        self.outline = 'notSet'
        self.shadow = 'notSet'
        self.condense = 'notSet'
        self.extend = 'notSet'
        self.underline = 'notSet'
        self.vertAlign = 'notSet'
        self.color = 'notSet'
        self.scheme = 'notSet'

    def set_name(self, name):
        self.name = name

    def set_charset(self, charset):
        if not isinstance(charset, int):
            raise TypeError('charset参数只能接收int类型。')
        self.charset = charset

    def set_size(self, size):
        self.size = size

    def set_bold(self, on_off):
        self.bold = on_off

    def set_italic(self, on_off):
        self.italic = on_off

    def set_strike(self, on_off):
        self.strike = on_off

    def set_outline(self, on_off):
        self.outline = on_off

    def set_shadow(self, on_off):
        self.shadow = on_off

    def set_condense(self, on_off):
        self.condense = on_off

    def set_extend(self, on_off):
        self.extend = on_off

    def set_color(self, color):
        self.color = get_color_code(color)

    def set_underline(self, option):
        if option not in self._LINE_STYLES:
            raise ValueError(f'option参数只能是{self._LINE_STYLES}其中之一。')
        self.underline = option

    def set_vertAlign(self, option):
        if option not in self._VERT_ALIGNS:
            raise ValueError(f'option参数只能是{self._VERT_ALIGNS}其中之一。')
        self.vertAlign = option

    def set_scheme(self, option):
        if option not in self._SCHEMES:
            raise ValueError(f'option参数只能是{self._SCHEMES}其中之一。')
        self.scheme = option


class CellBorder(object):
    _LINE_STYLES = ('dashDot', 'dashDotDot', 'dashed', 'dotted', 'double', 'hair', 'medium', 'mediumDashDot',
                    'mediumDashDotDot', 'mediumDashed', 'slantDashDot', 'thick', 'thin', None)

    def __init__(self):
        self.start = 'notSet'
        self.end = 'notSet'
        self.left = 'notSet'
        self.right = 'notSet'
        self.top = 'notSet'
        self.bottom = 'notSet'
        self.diagonal = 'notSet'
        self.vertical = 'notSet'
        self.horizontal = 'notSet'
        self.horizontal = 'notSet'
        self.outline = 'notSet'
        self.diagonalUp = 'notSet'
        self.diagonalDown = 'notSet'

    def set_start(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.start = Side(style=style, color=get_color_code(color))

    def set_end(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.end = Side(style=style, color=get_color_code(color))

    def set_left(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.left = Side(style=style, color=get_color_code(color))

    def set_right(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.right = Side(style=style, color=get_color_code(color))

    def set_top(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.top = Side(style=style, color=get_color_code(color))

    def set_bottom(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.bottom = Side(style=style, color=get_color_code(color))

    def set_quad(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.bottom = self.top = self.right = self.left = Side(style=style, color=get_color_code(color))

    def set_diagonal(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.diagonal = Side(style=style, color=get_color_code(color))

    def set_vertical(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.vertical = Side(style=style, color=get_color_code(color))

    def set_horizontal(self, style, color):
        if style not in self._LINE_STYLES:
            raise ValueError(f'style参数只能是{self._LINE_STYLES}之一。')
        self.horizontal = Side(style=style, color=get_color_code(color))

    def set_outline(self, on_off):
        self.outline = on_off

    def set_diagonalDown(self, on_off):
        self.diagonalDown = on_off

    def set_diagonalUp(self, on_off):
        self.diagonalUp = on_off


class CellAlignment(object):
    _horizontal_alignments = ('general', 'left', 'center', 'right', 'fill', 'justify', 'centerContinuous',
                              'distributed', None)
    _vertical_alignments = ('top', 'center', 'bottom', 'justify', 'distributed', None)

    def __init__(self):
        self.horizontal = 'notSet'
        self.vertical = 'notSet'
        self.indent = 'notSet'
        self.relativeIndent = 'notSet'
        self.justifyLastLine = 'notSet'
        self.readingOrder = 'notSet'
        self.textRotation = 'notSet'
        self.wrapText = 'notSet'
        self.shrinkToFit = 'notSet'

    def set_horizontal(self, horizontal):
        if horizontal not in self._horizontal_alignments:
            raise ValueError(f'horizontal参数必须是{self._horizontal_alignments}其中之一。')
        self.horizontal = horizontal

    def set_vertical(self, vertical):
        if vertical not in self._vertical_alignments:
            raise ValueError(f'horizontal参数必须是{self._vertical_alignments}其中之一。')
        self.vertical = vertical

    def set_indent(self, indent):
        if not (isinstance(indent, int) and 0 <= indent <= 255):
            raise ValueError('value参数必须在0到255之间。')
        self.indent = indent

    def set_relativeIndent(self, indent):
        if not (isinstance(indent, int) and -255 <= indent <= 255):
            raise ValueError('value参数必须在-255到255之间。')
        self.relativeIndent = indent

    def set_justifyLastLine(self, on_off):
        self.justifyLastLine = on_off

    def set_readingOrder(self, value):
        if not (isinstance(value, int) and value >= 0):
            raise ValueError('value参数必须不小于0。')
        self.readingOrder = value

    def set_textRotation(self, value):
        if not (0 <= value <= 180 or value == 255):
            raise ValueError('value必须在0到180之间。')
        self.textRotation = value

    def set_wrapText(self, on_off):
        self.wrapText = on_off

    def set_shrinkToFit(self, on_off):
        self.shrinkToFit = on_off


class CellGradientFill(object):
    def __init__(self):
        self.type = 'notSet'
        self.degree = 'notSet'
        self.left = 'notSet'
        self.right = 'notSet'
        self.top = 'notSet'
        self.bottom = 'notSet'
        self.stop = 'notSet'

    def set_type(self, name):
        if name not in ('linear', 'path'):
            raise ValueError("name参数只能是 'linear', 'path' 之一。")
        self.type = name

    def set_degree(self, value):
        self.degree = value

    def set_left(self, value):
        self.left = value

    def set_right(self, value):
        self.right = value

    def set_top(self, value):
        self.top = value

    def set_bottom(self, value):
        self.bottom = value

    def set_stop(self, values):
        self.stop = values


class CellPatternFill(object):
    _FILES = ('none', 'solid', 'darkDown', 'darkGray', 'darkGrid', 'darkHorizontal', 'darkTrellis', 'darkUp',
              'darkVertical', 'gray0625', 'gray125', 'lightDown', 'lightGray', 'lightGrid', 'lightHorizontal',
              'lightTrellis', 'lightUp', 'lightVertical', 'mediumGray', None)

    def __init__(self):
        self.patternType = 'notSet'
        self.fgColor = 'notSet'
        self.bgColor = 'notSet'

    def set_patternType(self, name):
        if name not in self._FILES:
            raise ValueError(f'name参数只能是{self._FILES}其中之一。')
        self.patternType = name

    def set_fgColor(self, color):
        self.fgColor = get_color_code(color)
        if self.patternType == 'notSet':
            self.patternType = 'solid'

    def set_bgColor(self, color):
        self.bgColor = get_color_code(color)
        if self.patternType == 'notSet':
            self.patternType = 'solid'


class CellNumberFormat(object):
    def __init__(self):
        self.format = 'notSet'

    def set_format(self, string):
        if string is None:
            string = 'General'
        self.format = string


class CellProtection(object):
    def __init__(self):
        self.hidden = 'notSet'
        self.locked = 'notSet'

    def set_hidden(self, on_off):
        self.hidden = on_off

    def set_locked(self, on_off):
        self.locked = on_off


class CellStyleCopier(object):
    def __init__(self, from_cell):
        """
        :param from_cell: 被复制单元格对象
        """
        self._style = copy(from_cell._style)
        self._font = copy(from_cell.font)
        self._border = copy(from_cell.border)
        self._fill = copy(from_cell.fill)
        self._number_format = copy(from_cell.number_format)
        self._protection = copy(from_cell.protection)
        self._alignment = copy(from_cell.alignment)

    def to_cell(self, cell):
        cell._style = self._style
        cell.alignment = self._alignment
        cell.font = self._font
        cell.border = self._border
        cell.fill = self._fill
        cell.number_format = self._number_format
        cell.protection = self._protection


def get_color_code(color):
    if color is None:
        return '000000'
    if isinstance(color, Color):
        return color
    __COLORS__ = {
        'white': 'FFFFFF',
        'black': '000000',
        'red': 'FF0000',
        'green': '7FB80E',
        'blue': '009AD6',
        'purple': '8552A1',
        'yellow': 'FFFF00',
        'orange': 'F58220'
    }
    color = str(color)
    if ',' in color:
        color = color.replace(' ', '').lstrip('(').rstrip(')')
        RGB = color.split(',')
        color = ''
        for i in RGB:
            num = int(i)
            color += str(hex(num))[-2:].replace('x', '0').upper()
        return color

    return __COLORS__.get(color, color).lstrip('#')


class NoneStyle(object):
    _instance_lock = Lock()

    def __init__(self):
        self._font = Font()
        self._border = Border()
        self._alignment = Alignment()
        self._fill = PatternFill()
        self._number_format = 'General'
        self._protection = Protection()

    def __new__(cls, *args, **kwargs):
        if not hasattr(NoneStyle, "_instance"):
            with NoneStyle._instance_lock:
                if not hasattr(NoneStyle, "_instance"):
                    NoneStyle._instance = object.__new__(cls)
        return NoneStyle._instance

    def to_cell(self, cell, replace=True):
        cell.font = self._font
        cell.border = self._border
        cell.alignment = self._alignment
        cell.fill = self._fill
        cell.protection = self._protection
        cell.number_format = 'General'



# -*- coding:utf-8 -*-
from pathlib import Path
from time import sleep

from .base import OriginalRecorder


class ByteRecorder(OriginalRecorder):
    _SUPPORTS = ('any',)
    __END = (0, 2)

    def __init__(self, path=None, cache_size=1000):
        super().__init__(path, cache_size)
        self._type = 'byte'

    def add_data(self, data, seek=None):
        while self._pause_add:  # 等待其它线程写入结束
            sleep(.02)

        if not isinstance(data, bytes):
            raise TypeError('只能接受bytes类型数据。')
        if seek is not None and not (isinstance(seek, int) and seek >= 0):
            raise ValueError('seek参数只能接受None或大于等于0的整数。')

        self._data.append((data, seek))
        self._data_count += 1

        if 0 < self.cache_size <= self._data_count:
            self.record()

    def _record(self):
        if not self._file_exists and not Path(self.path).exists():
            with open(self.path, 'wb'):
                pass

        with open(self.path, 'rb+') as f:
            previous = None
            for i in self._data:
                loc = ByteRecorder.__END if i[1] is None else (i[1], 0)
                if not (previous == loc == ByteRecorder.__END):
                    f.seek(loc[0], loc[1])
                    previous = loc
                f.write(i[0])



# -*- coding:utf-8 -*-
from csv import reader as csv_reader, writer as csv_writer
from pathlib import Path
from time import sleep

from openpyxl.reader.excel import load_workbook

from .base import BaseRecorder
from .cell_style import NoneStyle
from .setter import RecorderSetter, set_csv_header
from .tools import (ok_list_str, process_content_xlsx, process_content_json, get_key_cols,
                    get_csv, parse_coord, do_nothing, Header, get_wb, get_ws,
                    get_real_coord, is_sigal_data, is_1D_data, get_real_col, data2ws, styles2ws, get_real_row)


class Recorder(BaseRecorder):
    _SUPPORTS = ('csv', 'xlsx', 'txt', 'jsonl', 'json')

    def __init__(self, path=None, cache_size=1000):
        self._header = {None: None}
        self._methods = {'xlsx': self._to_xlsx_fast,
                         'csv': self._to_csv_fast,
                         'txt': self._to_txt_fast,
                         'jsonl': self._to_jsonl_fast,
                         'json': self._to_json_fast,
                         'addData': not_type,
                         # fast模式下非添加data使用的方法
                         'setImg': do_nothing,
                         'setLink': do_nothing,
                         'setStyle': do_nothing,
                         'setHeight': do_nothing,
                         'setWidth': do_nothing,
                         # 写入不同类型数据时使用的方法
                         'img': img2ws,
                         'link': link2ws,
                         'style': styles2ws,
                         'height': height2ws,
                         'width': width2ws,
                         'data': data2ws}
        super().__init__(path=path, cache_size=cache_size)
        self._data = {}
        self._delimiter = ','  # csv文件分隔符
        self._quote_char = '"'  # csv文件引用符
        self._follow_styles = False
        self._row_height = None
        self._styles = None
        self._header_row = {None: 1}
        self._fast = True
        self._link_style = None
        self.data_col = 1

    def _set_methods(self, file_type):
        self._methods[file_type] = getattr(self, f'_to_{file_type}_fast')
        if file_type == 'xlsx':
            self._methods['setImg'] = self._set_img
            self._methods['setLink'] = self._set_link
            self._methods['setStyle'] = self._set_styles
            self._methods['setHeight'] = self._set_row_height
            self._methods['setWidth'] = self._set_col_width
            self._methods['addData'] = self._add_data_any
        else:
            self._methods['setImg'] = do_nothing
            self._methods['setLink'] = do_nothing
            self._methods['setStyle'] = do_nothing
            self._methods['setHeight'] = do_nothing
            self._methods['setWidth'] = do_nothing
            self._methods['addData'] = self._add_data_txt

    @property
    def set(self):
        if self._setter is None:
            self._setter = RecorderSetter(self)
        return self._setter

    @property
    def delimiter(self):
        return self._delimiter

    @property
    def quote_char(self):
        return self._quote_char

    @property
    def header(self):
        if self.type not in ('csv', 'xlsx'):
            raise TypeError('header属性只支持csv和xlsx类型文件。')
        return get_header(self)

    def add_data(self, data, coord=None, table=None):
        coord = parse_coord(coord, self.data_col)
        data, data_len = self._handle_data(data, coord)
        self._add(data, table,
                  True if self._fast and coord[0] else False,  # xlsx不需要fast模式
                  data_len, self._methods['addData'])

    def _handle_data(self, data, coord):
        if is_sigal_data(data):
            data = {'type': 'data', 'data': [self._make_final_data(self, (data,))], 'coord': coord}
            data_len = 1
        elif not data:
            data = {'type': 'data', 'data': [self._make_final_data(self, tuple())], 'coord': coord}
            data_len = 1
        elif is_1D_data(data):
            data = {'type': 'data', 'data': [self._make_final_data(self, data)], 'coord': coord}
            data_len = 1
        else:  # 二维数组
            data = {'type': 'data', 'coord': coord,
                    'data': [self._make_final_data(self, (d,)) if is_sigal_data(d)
                             else self._make_final_data(self, d) for d in data]}
            data_len = len(data)
        return data, data_len

    def _add(self, data, table, to_slow, num, add_method):
        while self._pause_add:  # 等待其它线程写入结束
            sleep(.02)

        if to_slow:
            self._slow_mode()

        if table is None:
            table = self._table
        elif table is True:
            table = None

        add_method(data, table)

        self._data_count += num
        if 0 < self.cache_size <= self._data_count:
            self.record()

    def _add_data_any(self, data, table):
        if (self._data.get(table, 0) != 0 and self._data[table]
                and data['coord'] == self._data[table][-1]['coord']
                and self._data[table][-1]['type'] == 'data'):
            self._data[table][-1]['data'].extend(data['data'])
        else:
            self._data.setdefault(table, []).append(data)

    def _add_data_txt(self, data, table):
        if self._data.get(None, None) and data['coord'] == self._data[None][-1]['coord']:
            self._data[table][-1]['data'].extend(data['data'])
        else:
            self._data.setdefault(None, []).append(data)

    def _add_others(self, data, table):
        self._data.setdefault(table, []).append(data)

    def _set_link(self, coord, link, content=None, table=None):
        self._add({'type': 'link', 'link': link, 'content': content,
                   'coord': parse_coord(coord, self.data_col)}, table, self._fast, 1, self._add_others)

    def _set_img(self, coord, img_path, width=None, height=None, table=None):
        self._add({'type': 'img', 'imgPath': img_path, 'width': width, 'height': height,
                   'coord': parse_coord(coord, self.data_col)}, table, self._fast, 1, self._add_others)

    def _set_styles(self, coord, styles, replace=True, table=None):
        if isinstance(coord, str):
            if ':' in coord:
                real, coord = coord, (1, 1)
            elif coord.isdigit() or (coord[0] == '-' and coord[1:].isdigit()):
                real, coord = int(coord), (1, 1)
            else:
                real, coord = None, parse_coord(coord, self.data_col)
        elif isinstance(coord, int):
            real, coord = coord, (1, 1)
        else:
            real, coord = None, parse_coord(coord, self.data_col)
        self._add({'type': 'style', 'mode': 'replace' if replace else 'cover', 'real_coord': real,
                   'styles': styles, 'coord': coord}, table, self._fast, 1, self._add_others)

    def _set_row_height(self, row, height, table=None):
        if not row:
            raise ValueError('row不能为0或None。')
        self._add({'type': 'height', 'row': row, 'height': height}, table, self._fast, 1, self._add_others)

    def _set_col_width(self, col, width, table=None):
        if not col:
            raise ValueError('col不能为0或None。')
        self._add({'type': 'width', 'col': col, 'width': width}, table, self._fast, 1, self._add_others)

    def set_link(self, coord, link, content=None, table=None):
        self._methods['setLink'](coord, link, content, table)

    def set_img(self, coord, img_path, width=None, height=None, table=None):
        self._methods['setImg'](coord, img_path, width, height, table)

    def set_styles(self, coord, styles, replace=True, table=None):
        self._methods['setStyle'](coord, styles, replace, table)

    def set_row_height(self, row, height, table=None):
        self._methods['setHeight'](row, height, table)

    def set_col_width(self, col, width, table=None):
        self._methods['setWidth'](col, width, table)

    def rows(self, key_cols=True, sign_col=True, is_header=False,
             signs=None, deny_sign=False, count=None, begin_row=None):
        if not self._path or not Path(self._path).exists():
            raise RuntimeError('未指定文件路径或文件不存在。')
        if self.type == 'xlsx':
            wb = load_workbook(self.path, data_only=True, read_only=True)
            if self.table and self.table not in [i.title for i in wb.worksheets]:
                raise RuntimeError(f'xlsx文件未包含指定工作表：{self.table}')
            ws = wb[self.table] if self.table else wb.active
            if ws.max_column is None:  # 遇到过read_only时无法获取列数的文件
                wb.close()
                wb = load_workbook(self.path, data_only=True)
                ws = wb[self.table] if self.table else wb.active

            method = get_xlsx_rows

        elif self.type == 'csv':
            ws = None
            method = get_csv_rows
        else:
            raise RuntimeError('rows()方法只支持xlsx和csv格式。')

        header = get_header(self, ws)
        if not begin_row:
            begin_row = self._header_row.get(self.table, self._header_row[None]) + 1

        if sign_col is not True:
            sign_col = header.get_num(sign_col, is_header=is_header) or 1
        if not isinstance(signs, (list, tuple, set)):
            signs = (signs,)
        key_cols = get_key_cols(key_cols, header, is_header)

        return method(self, header=header, key_cols=key_cols, begin_row=begin_row, sign_col=sign_col,
                      sign=signs, deny_sign=deny_sign, count=count, ws=ws)

    def _record(self):
        self._methods[self.type]()
        if not self._fast:
            self._fast_mode()

    def _fast_mode(self):
        self._methods['csv'] = self._to_csv_fast
        self._methods['txt'] = self._to_txt_fast
        self._methods['json'] = self._to_json_fast
        self._methods['jsonl'] = self._to_jsonl_fast
        self._fast = True

    def _slow_mode(self):
        self._methods['csv'] = self._to_csv_slow
        self._methods['txt'] = self._to_txt_slow
        self._methods['json'] = self._to_json_slow
        self._methods['jsonl'] = self._to_jsonl_slow
        self._fast = False

    def _to_xlsx_fast(self):
        wb, new_file = get_wb(self)
        tables = wb.sheetnames
        rewrite_method = 'make_num_dict_rewrite' if self._auto_new_header else 'make_num_dict'

        for table, data in self._data.items():
            ws, new_sheet = get_ws(wb, table, tables, new_file)
            new_file = False
            if table is None:
                if ws.title not in self._header:
                    self._header[ws.title] = self._header[None]
                if ws.title not in self._header_row:
                    self._header_row[ws.title] = self._header_row[None]

            begin_row = True
            if new_sheet:
                begin_row = handle_new_sheet(self, ws, data)
            elif self._header.get(ws.title, None) is None:
                self._header[ws.title] = Header([c.value for c in ws[self._header_row[ws.title]]])

            header = self._header[ws.title]
            rewrite = False
            if not begin_row and not data[0]['coord'][0]:
                cur = data[0]
                rewrite = self._methods[cur['type']](
                    **{'recorder': self,
                       'ws': ws,
                       'data': cur,
                       'coord': (1, get_real_col(cur.get('coord', (1, 1)), ws.max_column)),
                       'new_row': not cur.get('coord', (1, 1))[0],
                       'header': header,
                       'rewrite': rewrite,
                       'rewrite_method': rewrite_method})
                data = data[1:]

            for cur in data:
                rewrite = self._methods[cur['type']](
                    **{'recorder': self,
                       'ws': ws,
                       'data': cur,
                       'coord': get_real_coord(cur.get('coord', (1, 1)), ws.max_row, ws.max_column),
                       'new_row': not cur.get('coord', (1, 1))[0],
                       'header': header,
                       'rewrite': rewrite,
                       'rewrite_method': rewrite_method})

            if rewrite:
                for c in range(1, ws.max_column + 1):
                    ws.cell(self._header_row[ws.title], c, value=header[c])

        wb.save(self.path)
        wb.close()

    def _to_csv_fast(self):
        file, new_csv = get_csv(self)
        writer = csv_writer(file, delimiter=self.delimiter, quotechar=self.quote_char)
        get_and_set_csv_header(self, new_csv, file, writer)
        rewrite_method = 'make_insert_list_rewrite' if self._auto_new_header else 'make_insert_list'

        rewrite = False
        header = self._header[None]
        for d in self._data[None]:
            col = 1 if d['coord'][1] == 1 else get_real_col(d['coord'][1],
                                                            len(header) if self._header_row[None] > 0 else 1)
            for data in d['data']:
                data, rewrite = header.__getattribute__(rewrite_method)(data, 'csv', rewrite)
                data = [None] * (col - 1) + data
                writer.writerow(data)
        file.close()

        if rewrite:
            set_csv_header(self, self._header[None], self._header_row[None])

    def _to_csv_slow(self):
        file, new_csv = get_csv(self)
        writer = csv_writer(file, delimiter=self.delimiter, quotechar=self.quote_char)
        get_and_set_csv_header(self, new_csv, file, writer)
        file.seek(0)
        reader = csv_reader(file, delimiter=self.delimiter, quotechar=self.quote_char)
        lines = list(reader)
        lines_count = len(lines)
        header = self._header[None]

        rewrite = False
        method = 'make_change_list_rewrite' if self._auto_new_header else 'make_change_list'
        for i in self._data[None]:
            data = i['data']
            row, col = get_real_coord(i['coord'], lines_count, len(header) if self._header_row[None] > 0 else 1)
            for r, da in enumerate(data, row):
                add_rows = r - lines_count
                if add_rows > 0:  # 若行数不够，填充行数
                    [lines.append([]) for _ in range(add_rows)]
                    lines_count += add_rows
                row_num = r - 1
                lines[row_num], rewrite = self._header[None].__getattribute__(method)(lines[row_num], da, col,
                                                                                      'csv', rewrite)

        if rewrite:
            [lines.append([]) for _ in range(self._header_row[None] - lines_count)]  # 若行数不够，填充行数
            lines[self._header_row[None] - 1] = list(header.num_key.values())

        file.close()
        writer = csv_writer(open(self.path, 'w', encoding=self.encoding, newline=''),
                            delimiter=self.delimiter, quotechar=self.quote_char)
        writer.writerows(lines)

    def _to_txt_fast(self):
        with open(self.path, 'a+', encoding=self.encoding) as f:
            all_data = []
            for data in self._data[None]:
                for d in data['data']:
                    all_data.append(' '.join(ok_list_str(d)))
            f.write('\n'.join(all_data) + '\n')

    def _to_jsonl_fast(self):
        from json import dumps
        with open(self.path, 'a+', encoding=self.encoding) as f:
            all_data = []
            for data in self._data[None]:
                for d in data['data']:
                    all_data.append(d if isinstance(d, str) else dumps(d))
            f.write('\n'.join(all_data) + '\n')

    def _to_json_fast(self):
        from json import load, dump
        if self._file_exists or Path(self.path).exists():
            with open(self.path, 'r', encoding=self.encoding) as f:
                json_data = load(f)
        else:
            json_data = []

        for i in self._data[None]:
            for data in i['data']:
                if isinstance(data, dict):
                    for k, d in data.items():
                        data[k] = process_content_json(d)
                    json_data.append(data)
                else:
                    json_data.append([process_content_json(d) for d in i])

        with open(self.path, 'w', encoding=self.encoding) as f:
            dump(json_data, f)

    def _to_txt_slow(self):
        if not self._file_exists and not Path(self.path).exists():
            with open(self.path, 'w', encoding=self.encoding):
                pass
        with open(self.path, 'r+', encoding=self.encoding) as f:
            lines = f.readlines()
            handle_txt_lines(self._data[None], lines, '\n', handle_jsonl_data)
            f.seek(0)
            f.writelines(lines)

    def _to_jsonl_slow(self):
        if not self._file_exists and not Path(self.path).exists():
            with open(self.path, 'w', encoding=self.encoding):
                pass
        with open(self.path, 'r+', encoding=self.encoding) as f:
            lines = f.readlines()
            handle_txt_lines(self._data[None], lines, '[]\n', handle_jsonl_data)
            f.seek(0)
            f.writelines(lines)

    def _to_json_slow(self):
        from json import load, dump
        if self._file_exists or Path(self.path).exists():
            with open(self.path, 'r', encoding=self.encoding) as f:
                lines = load(f)
        else:
            lines = []
        handle_txt_lines(self._data[None], lines, None, handle_json_data)
        with open(self.path, 'w', encoding=self.encoding) as f:
            dump(lines, f)


def handle_txt_lines(data_lst, lines, val, method):
    lines_len = len(lines)
    for data in data_lst:
        num = get_real_row(data['coord'][0], lines_len)
        data_end = num + len(data['data'])
        if lines_len < data_end:
            diff = data_end - lines_len - 1
            [lines.append(val) for _ in range(diff)]
            lines_len += diff
        for num, i in enumerate(data['data'], num - 1):
            method(lines, num, i)


def handle_txt_data(lines, num, data):
    lines[num] = ' '.join(ok_list_str(data)) + '\n'


def handle_jsonl_data(lines, num, data):
    from json import dumps
    lines[num] = data if isinstance(data, str) else dumps(data) + '\n'


def handle_json_data(lines, num, data):
    if isinstance(data, dict):
        for k, d in data.items():
            data[k] = process_content_json(d)
        lines[num] = data
    else:
        lines[num] = [process_content_json(d) for d in data]


def get_header(recorder, ws=None):
    header = recorder._header.get(recorder._table, None)
    if header is not None:
        return header
    if not recorder.path or not Path(recorder.path).exists():
        return None

    if recorder.type == 'xlsx':
        if not ws:
            wb = load_workbook(recorder.path)
            if not recorder.table:
                ws = wb.active
            elif recorder.table not in wb.sheetnames:
                wb.close()
                return Header()
            else:
                ws = wb[recorder.table]
        header_row = recorder._header_row.get(recorder.table, recorder._header_row[None])
        if header_row > ws.max_row:
            recorder._header[recorder.table] = Header()
        else:
            recorder._header[recorder.table] = Header(
                [i.value for i in ws[recorder._header_row.get(recorder.table, recorder._header_row[None])]])

        if not ws:
            wb.close()
        return recorder._header[recorder.table]

    if recorder.type == 'csv':
        from csv import reader
        with open(recorder.path, 'r', newline='', encoding=recorder.encoding) as f:
            u = reader(f, delimiter=recorder.delimiter, quotechar=recorder.quote_char)
            try:
                for _ in range(recorder._header_row[None]):
                    header = next(u)
            except StopIteration:  # 文件是空的
                header = []
        recorder._header[None] = Header(header)
        return recorder._header[None]


def handle_new_sheet(recorder, ws, data):
    if not recorder._header_row:
        return 0

    if recorder._header.get(ws.title, None) is not None:
        for c, h in recorder._header[ws.title].items():
            ws.cell(row=recorder._header_row[ws.title], column=c, value=h)
        begin_row = recorder._header_row

    else:
        data = get_first_dict(data)
        if data:
            header = Header([h for h in data.keys() if isinstance(h, str)])
            recorder._header[ws.title] = header
            for c, h in header.items():
                ws.cell(row=recorder._header_row[ws.title], column=c, value=h)
            begin_row = recorder._header_row

        else:
            recorder._header[ws.title] = Header()
            begin_row = 0

    return begin_row


def get_first_dict(data):
    if not data:
        return False
    elif data[0]['type'] == 'data' and data[0]['data'] and isinstance(data[0]['data'][0], dict):
        return data[0]['data'][0]


def get_xlsx_rows(recorder, header, key_cols, begin_row, sign_col, sign, deny_sign, count, ws):
    rows = ws.rows
    try:
        for _ in range(begin_row - 1):
            next(rows)
    except StopIteration:
        return []

    if sign_col is True or sign_col > ws.max_column:  # 获取所有行
        if count:
            rows = list(rows)[:count]
        if key_cols is True:  # 获取整行
            res = [header.make_row_data(ind, {col: cell.value for col, cell in enumerate(row, 1)})
                   for ind, row in enumerate(rows, begin_row)]
        else:  # 只获取对应的列
            res = [header.make_row_data(ind, {col: row[col - 1].value for col in key_cols})
                   for ind, row in enumerate(rows, begin_row)]

    else:  # 获取符合条件的行
        if count:
            res = get_xlsx_rows_with_count(key_cols, deny_sign, header, rows,
                                           begin_row, sign_col, sign, count)
        else:
            res = get_xlsx_rows_without_count(key_cols, deny_sign, header, rows, begin_row,
                                              sign_col, sign)

    ws.parent.close()
    return res


def get_csv_rows(recorder, header, key_cols, begin_row, sign_col, sign, deny_sign, count, ws):
    sign = ['' if i is None else str(i) for i in sign]
    begin_row -= 1
    res = []
    with open(recorder.path, 'r', encoding=recorder.encoding) as f:
        reader = csv_reader(f, delimiter=recorder.delimiter, quotechar=recorder.quote_char)
        lines = list(reader)
        if not lines:
            return res

        if sign_col is True:  # 获取所有行
            header_len = len(header)
            for ind, line in enumerate(lines[begin_row:count + 1 if count else None], begin_row + 1):
                if key_cols is True:  # 获取整行
                    if not line:
                        res.append(header.make_row_data(ind, {col: '' for col in range(1, header_len + 1)}))
                    else:
                        line_len = len(line)
                        x = max(header_len, line_len)
                        res.append(header.make_row_data(ind, {col: line[col - 1] if col <= line_len else ''
                                                              for col in range(1, x + 1)}))
                else:  # 只获取对应的列
                    x = len(line) + 1
                    res.append(header.make_row_data(ind, {col: line[col - 1] if col < x else '' for col in key_cols}))

        else:  # 获取符合条件的行
            sign_col -= 1
            if count:
                get_csv_rows_with_count(lines, begin_row, sign_col, sign, deny_sign,
                                        key_cols, res, header, count)
            else:
                get_csv_rows_without_count(lines, begin_row, sign_col, sign, deny_sign,
                                           key_cols, res, header)

    return res


def get_xlsx_rows_with_count(key_cols, deny_sign, header, rows, begin_row, sign_col, sign, count):
    got = 0
    res = []
    if key_cols is True:  # 获取整行
        if deny_sign:
            for ind, row in enumerate(rows, begin_row):
                if got == count:
                    break
                if row[sign_col - 1].value not in sign:
                    res.append(header.make_row_data(ind, {col: cell.value for col, cell in enumerate(row, 1)}))
                    got += 1
        else:
            for ind, row in enumerate(rows, begin_row):
                if got == count:
                    break
                if row[sign_col - 1].value in sign:
                    res.append(header.make_row_data(ind, {col: cell.value for col, cell in enumerate(row, 1)}))
                    got += 1

    else:  # 只获取对应的列
        if deny_sign:
            for ind, row in enumerate(rows, begin_row):
                if got == count:
                    break
                if row[sign_col - 1].value not in sign:
                    res.append(header.make_row_data(ind, {col: row[col - 1].value for col in key_cols}))
                    got += 1
        else:
            for ind, row in enumerate(rows, begin_row):
                if got == count:
                    break
                if row[sign_col - 1].value in sign:
                    res.append(header.make_row_data(ind, {col: row[col - 1].value for col in key_cols}))
                    got += 1
    return res


def get_xlsx_rows_without_count(key_cols, deny_sign, header, rows, begin_row, sign_col, sign):
    if key_cols is True:  # 获取整行
        if deny_sign:
            return [header.make_row_data(ind, {col: cell.value for col, cell in enumerate(row, 1)})
                    for ind, row in enumerate(rows, begin_row)
                    if row[sign_col - 1].value not in sign]
        else:
            return [header.make_row_data(ind, {col: cell.value for col, cell in enumerate(row, 1)})
                    for ind, row in enumerate(rows, begin_row)
                    if row[sign_col - 1].value in sign]

    else:  # 只获取对应的列
        if deny_sign:
            return [header.make_row_data(ind, {col: row[col - 1].value for col in key_cols})
                    for ind, row in enumerate(rows, begin_row)
                    if row[sign_col - 1].value not in sign]
        else:
            return [header.make_row_data(ind, {col: row[col - 1].value for col in key_cols})
                    for ind, row in enumerate(rows, begin_row)
                    if row[sign_col - 1].value in sign]


def get_csv_rows_with_count(lines, begin_row, sign_col, sign, deny_sign, key_cols, res, header, count):
    got = 0
    header_len = len(header)
    for ind, line in enumerate(lines[begin_row:], begin_row + 1):
        if got == count:
            break
        row_sign = '' if sign_col > len(line) - 1 else line[sign_col]
        if (row_sign not in sign) if deny_sign else (row_sign in sign):
            if key_cols is True:  # 获取整行
                if not line:
                    res.append(header.make_row_data(ind, {col: '' for col in range(1, header_len + 1)}))
                else:
                    line_len = len(line)
                    x = max(header_len, line_len)
                    res.append(header.make_row_data(ind, {col: line[col - 1] if col <= line_len else ''
                                                          for col in range(1, x + 1)}))
            else:  # 只获取对应的列
                x = len(line) + 1
                res.append(header.make_row_data(ind, {col: line[col - 1] if col < x else '' for col in key_cols}))
            got += 1


def get_csv_rows_without_count(lines, begin_row, sign_col, sign, deny_sign, key_cols, res, header):
    header_len = len(header)
    for ind, line in enumerate(lines[begin_row:], begin_row + 1):
        row_sign = '' if sign_col > len(line) - 1 else line[sign_col]
        if (row_sign not in sign) if deny_sign else (row_sign in sign):
            if key_cols is True:  # 获取整行
                if not line:
                    res.append(header.make_row_data(ind, {col: '' for col in range(1, header_len + 1)}))
                else:
                    line_len = len(line)
                    x = max(header_len, line_len)
                    res.append(header.make_row_data(ind, {col: line[col - 1] if col <= line_len else ''
                                                          for col in range(1, x + 1)}))
            else:  # 只获取对应的列
                x = len(line) + 1
                res.append(header.make_row_data(ind, {col: line[col - 1] if col < x else '' for col in key_cols}))


def get_and_set_csv_header(recorder, new_csv, file, writer):
    if not recorder._header_row:
        return

    if new_csv:
        if recorder._header[None]:
            for _ in range(recorder._header_row[None] - 1):
                writer.writerow([])
            writer.writerow(ok_list_str(recorder._header[None]))

        if recorder._header[None] is None and recorder._data_count:
            data = get_first_dict(recorder._data[None])
            if data:
                recorder._header[None] = Header([h for h in data.keys() if isinstance(h, str)])
            else:
                recorder._header[None] = Header()
            writer.writerow(ok_list_str(recorder._header[None]))
        else:
            recorder._header[None] = Header()

    elif recorder._header[None] is None:  # 从文件读取表头
        file.seek(0)
        reader = csv_reader(file, delimiter=recorder.delimiter, quotechar=recorder.quote_char)
        header = []
        try:
            for _ in range(recorder._header_row[None]):
                header = next(reader)
        except StopIteration:
            pass
        recorder._header[None] = Header(header)
        file.seek(2)


def link2ws(**kwargs):
    recorder = kwargs['recorder']
    data = kwargs['data']
    cell = kwargs['ws'].cell(*kwargs['coord'])
    has_link = bool(cell.hyperlink)
    cell.hyperlink = data['link']
    if data['content'] is not None:
        cell.value = process_content_xlsx(data['content'])
    if data['link']:
        if recorder._link_style:
            recorder._link_style.to_cell(cell, replace=False)
    elif has_link:
        NoneStyle().to_cell(cell, replace=False)


def img2ws(**kwargs):
    row, col = kwargs['coord']
    data = kwargs['data']
    ws = kwargs['ws']
    from openpyxl.drawing.image import Image
    img = Image(data['imgPath'])
    width, height = data['width'], data['height']
    if width and height:
        img.width = width
        img.height = height
    elif width:
        img.height = int(img.height * (width / img.width))
        img.width = width
    elif height:
        img.width = int(img.width * (height / img.height))
        img.height = height
    # ws.add_image(img, (row, Header._NUM_KEY[col]))
    ws.add_image(img, f'{Header._NUM_KEY[col]}{row}')


def width2ws(**kwargs):
    col, width = kwargs['data']['col'], kwargs['data']['width']
    ws = kwargs['ws']
    if isinstance(col, int):
        col = Header._NUM_KEY[col]
    for c in col.split(':'):
        if c.isdigit():
            c = Header._NUM_KEY[int(c)]
        ws.column_dimensions[c].width = width


def height2ws(**kwargs):
    row, height = kwargs['data']['row'], kwargs['data']['height']
    ws = kwargs['ws']
    if isinstance(row, int):
        ws.row_dimensions[row].height = height
    elif isinstance(row, str):
        for r in row.split(':'):
            ws.row_dimensions[int(r)].height = height


def not_type(*keys):
    raise RuntimeError('添加数据前请先指定文件路径或类型。')



# -*- coding:utf-8 -*-
from io import TextIOWrapper
from pathlib import Path
from typing import Union, Tuple, Any, Optional, List, Dict, Iterable, Literal

from openpyxl.workbook import Workbook
from openpyxl.worksheet.worksheet import Worksheet

from .base import BaseRecorder
from .cell_style import CellStyleCopier
from .recorder import Recorder

REWRITE_METHOD = Literal['make_num_dict_rewrite', 'make_num_dict']


def line2ws(ws: Worksheet, header: Header, row: int, col: int, data: Union[dict, list], rewrite_method: REWRITE_METHOD,
            rewrite: bool) -> bool:
    """把一行数据写入数据表，不设置样式
    :param ws: Worksheet对象
    :param header: Header对象
    :param row: 行号
    :param col: 列序号
    :param data: 行数据
    :param rewrite_method: 'make_num_dict_rewrite'或'make_num_dict'
    :param rewrite: 是否重写表头
    :return: 是否重写表头
    """
    ...


def line2ws_follow(ws: Worksheet, header: Header, row: int, col: int, data: Union[dict, list],
                   rewrite_method: REWRITE_METHOD, rewrite: bool, styles: Dict[int, CellStyleCopier],
                   height: Optional[float], new_row: bool) -> bool:
    """把一行数据写入数据表，并设置样式
    :param ws: Worksheet对象
    :param header: Header对象
    :param row: 行号
    :param col: 列序号
    :param data: 行数据
    :param rewrite_method: 'make_num_dict_rewrite'或'make_num_dict'
    :param rewrite: 是否重写表头 
    :param styles: 样式对象了列表
    :param height: 行高，仅新行时有效
    :param new_row: 是否新行
    :return: 是否重写表头
    """
    ...


def data2ws(recorder: Recorder, ws: Worksheet, data: dict, coord: Tuple[int, int],
            header: Header, rewrite: bool, rewrite_method: REWRITE_METHOD) -> bool:
    """数据写入数据表
    :param recorder: Recorder对象
    :param ws: Worksheet对象
    :param data: 标准数据 {'type': 'data', 'data': [(1, 2, 3, 4)], 'coord': (0, 1)}
    :param coord: 要写入的坐标
    :param header: Header对象
    :param rewrite: 是否重写表头
    :param rewrite_method: 'make_num_dict_rewrite'或'make_num_dict'
    :return: 是否重写表头
    """
    ...


def data2ws_follow(recorder: Recorder, ws: Worksheet, data: dict, coord: Tuple[int, int],
                   header: Header, rewrite: bool, rewrite_method: REWRITE_METHOD) -> None:
    """数据写入数据表，跟随上一行样式
    :param recorder: Recorder对象
    :param ws: Worksheet对象
    :param data: 标准数据 {'type': 'data', 'data': [(1, 2, 3, 4)], 'coord': (0, 1)}
    :param coord: 要写入的坐标
    :param header: Header对象
    :param rewrite: 是否重写表头
    :param rewrite_method: 'make_num_dict_rewrite'或'make_num_dict'
    :return: 是否重写表头
    """
    ...


def data2ws_style(recorder: Recorder, ws: Worksheet, data: dict, coord: Tuple[int, int],
                  header: Header, rewrite: bool, rewrite_method: REWRITE_METHOD) -> None:
    """数据写入数据表，并设置指定样式
    :param recorder: Recorder对象
    :param ws: Worksheet对象
    :param data: 标准数据 {'type': 'data', 'data': [(1, 2, 3, 4)], 'coord': (0, 1)}
    :param coord: 要写入的坐标
    :param header: Header对象
    :param rewrite: 是否重写表头
    :param rewrite_method: 'make_num_dict_rewrite'或'make_num_dict'
    :return: 是否重写表头
    """
    ...


def styles2new_row(ws: Worksheet, styles: Iterable, height: float, row: int) -> None:
    """
    :param ws: Worksheet对象
    :param styles: Style对象租车的列表
    :param height: 行高
    :param row: 行号
    :return: None
    """
    ...


def styles2ws(**kwargs) -> None:
    """把样式写入数据表"""
    ...


def remove_end_Nones(in_list: list) -> list:
    """去除列表后面所有None
    :param in_list: 要处理的list
    :return: 处理后的列表
    """
    ...


class BaseHeader(object):
    _NUM_KEY: dict = ...
    _KEY_NUM: dict = ...
    _CONTENT_FUNCS: dict = ...

    @property
    def key_num(self) -> Dict[str, int]:
        """{str: int}格式的表头数据"""
        ...

    @property
    def num_key(self) -> Dict[int, str]:
        """{int: str}格式的表头数据"""
        ...

    def __iter__(self): ...


class Header(BaseHeader):

    def __init__(self, header: Union[list, tuple] = None): ...

    def __getitem__(self, item: Union[int, str]): ...

    def __len__(self) -> int: ...

    def values(self):
        """"""
        ...

    def items(self):
        """"""
        ...

    def make_row_data(self, row: int, row_values: dict, None_val: Optional[''] = None) -> RowData:
        """生成RowData对象
        :param row: 行号
        :param row_values: {列序号: 值}
        :param None_val: 空值是None还是''
        :return: RowData对象
        """
        ...

    def make_insert_list(self, data, file_type: Optional[str], rewrite: bool) -> Tuple[list, bool]:
        """生成写入文件list格式的新行数据
        :param data: 待处理行数据
        :param file_type: 文件类型，用于选择处理方法
        :param rewrite: 只用于对齐参数
        :return: 处理后的行数据
        """
        ...

    def make_change_list(self, line_data, data, col: int,
                         file_type: Optional[str], rewrite: bool) -> Tuple[list, bool]:
        """生产写入文件list格式的原有行数据
        :param line_data: 原有行数据
        :param data: 待处理行数据
        :param col: 要写入的列
        :param file_type: 文件类型，用于选择处理方法
        :param rewrite: 只用于对齐参数
        :return: (处理后的行数据, 是否重写表头)
        """
        ...

    def make_insert_list_rewrite(self, data, file_type: Optional[str], rewrite: bool) -> Tuple[list, bool]:
        """生产写入文件list格式的新行数据
        :param data: 待处理行数据
        :param rewrite: 是否需要重写表头
        :param file_type: 文件类型，用于选择处理方法
        :return: (处理后的行数据, 是否重写表头)
        """
        ...

    def make_change_list_rewrite(self, line_data, data, col: int, file_type, rewrite: bool) -> Tuple[list, bool]:
        """生产写入文件list格式的原有行数据
        :param line_data: 原有行数据
        :param data: 待处理行数据
        :param col: 要写入的列
        :param rewrite: 是否需要重写表头
        :param file_type: 文件类型，用于选择处理方法
        :return: (处理后的行数据, 是否重写表头)
        """
        ...

    def make_num_dict(self, *keys) -> Tuple[Dict[int, Any], bool, int]:
        """生成{int: val}的行数据，不考虑是否重写表头
        :return: (处理后的行数据, 是否重写表头, 表头长度)
        """
        ...

    def make_num_dict_rewrite(self, *keys) -> Tuple[Dict[int, Any], bool, int]:
        """生成{int: val}的行数据，虑是否重写表头
        :return: (处理后的行数据, 是否重写表头, 表头长度)
        """
        ...

    def get_key(self, num: int) -> Union[str, int]:
        """返回指定列序号对应的header key，如为None返回列序号
        :param num: 列序号
        :return: 表头值或列序号
        """
        ...

    def get_num(self, col: Union[int, str], is_header: bool = True) -> Optional[int]:
        """获取某列序号
        :param col: 列号、列名、表头值
        :param is_header: 当col为str时，是header的key还是列名
        :return: 列号int
        """
        ...


class ZeroHeader(Header):
    _OBJ: ZeroHeader = ...


class RowData(dict):
    header: Header = ...
    row: int = ...
    None_val: Optional[''] = ...

    def __init__(self, row: int, header: Header, None_val: Optional[''], seq: dict): ...

    def val(self, key: Union[int, str], is_header: bool = False, coord: bool = False) -> Any:
        """当前行获取指定列的值
        :param key: 为int时表示列序号，为str时表示列号或header key
        :param is_header: 为str时是header key还是列号
        :param coord: 为True时返回结果带坐标
        :return: coord为False时返回指定列的值，为Ture时返回(坐标, 值)
        """
        ...

    def col(self, key: str, num: bool = True) -> Union[str, int]:
        """获取指定表头项数据所在列
        :param key: 表头项
        :param num: 为True时返回列序号，否则返回列号
        :return: num: 为True时返回int格式列序号，否则返回str格式列号
        """
        ...


def align_csv(path: Union[str, Path], encoding: str = 'utf-8', delimiter: str = ',', quotechar: str = '"') -> None:
    """补全csv文件，使其每行列数一样多，用于pandas读取时避免出错
    :param path: 要处理的文件路径
    :param encoding: 文件编码
    :param delimiter: 分隔符
    :param quotechar: 引用符
    :return: None
    """
    ...


def get_usable_path(path: Union[str, Path], is_file: bool = True, parents: bool = True) -> Path:
    """检查文件或文件夹是否有重名，并返回可以使用的路径
    :param path: 文件或文件夹路径
    :param is_file: 目标是文件还是文件夹
    :param parents: 是否创建目标路径
    :return: 可用的路径，Path对象
    """
    ...


def make_valid_name(full_name: str) -> str:
    """获取有效的文件名
    :param full_name: 文件名
    :return: 可用的文件名
    """
    ...


def get_long(txt: str) -> int:
    """返回字符串中字符个数（一个汉字是2个字符）
    :param txt: 字符串
    :return: 字符个数
    """
    ...


def parse_coord(coord: Union[int, str, list, tuple, None],
                data_col: int = 1) -> Tuple[Optional[int], Optional[int]]:
    """添加数据，每次添加一行数据，可指定坐标、列号或行号
    coord只输入数字（行号）时，列号为self.data_col值，如 3；
    输入列号，或没有行号的坐标时，表示新增一行，列号为此时指定的，如'c'、',3'、(None, 3)、'None,3'；
    输入 'newline' 时，表示新增一行，列号为self.data_col值；
    输入行列坐标时，填写到该坐标，如'a3'、'3,1'、(3,1)、[3,1]；
    输入的行号可以是负数（列号不可以），代表从下往上数，-1是倒数第一行，如'a-3'、(-3, 3)
    :param coord: 坐标、列号、行号
    :param data_col: 列号，用于只传入行号的情况
    :return: 坐标tuple：(行, 列)坐标中的None表示新行或列
    """
    ...


def process_content_xlsx(content: Any) -> Union[None, int, str, float]:
    """处理单个单元格要写入的数据
    :param content: 未处理的数据内容
    :return: 处理后的数据
    """
    ...


def process_content_json(content: Any) -> Union[None, int, str, float]:
    """处理单个单元格要写入的数据
    :param content: 未处理的数据内容
    :return: 处理后的数据
    """
    ...


def process_content_str(content: Any) -> str:
    """处理单个单元格要写入的数据，以str格式输出
    :param content: 未处理的数据内容
    :return: 处理后的数据
    """
    ...


def process_nothing(content: Any) -> Any:
    """不处理直接返回数据"""
    ...


def ok_list_xlsx(data_list: Iterable) -> list:
    """处理列表中数据使其符合保存规范
    :param data_list: 数据列表
    :return: 处理后的列表
    """
    ...


def ok_list_str(data_list: Iterable) -> list:
    """处理列表中数据使其符合保存规范，所有数据都是str格式
    :param data_list: 数据列表
    :return: 处理后的列表
    """
    ...


def ok_list_db(data_list: Iterable) -> list:
    """处理列表中数据使其符合保存规范
    :param data_list: 数据列表
    :return: 处理后的列表
    """
    ...


def get_real_row(row: int, max_row: int):
    """获取返回真正写入文件的行号
    :param row: 输入的行号
    :param max_row: 最大行号
    :return: 真正的行号
    """
    ...


def get_real_col(col: int, max_col: int):
    """获取返回真正写入文件的列序号
    :param col: 输入的列序号
    :param max_col: 最大列号
    :return: 真正的列序号
    """
    ...


def get_real_coord(coord: Union[tuple, list],
                   max_row: int,
                   max_col: Union[int, Worksheet]) -> Tuple[int, int]:
    """返回真正写入文件的坐标
    :param coord: 已初步格式化的坐标，如(1, 2)、(None, 3)、(-3, -2)
    :param max_row: 文件最大行
    :param max_col: 文件最大列
    :return: 真正写入文件的坐标，tuple格式
    """
    ...


def make_final_data_simplify(recorder: BaseRecorder,
                             data: Union[list, tuple, dict, None]) -> Union[list, dict]:
    """将传入的数据转换为列表或字典形式，不添加前后列数据
    :param recorder: BaseRecorder对象
    :param data: 要处理的数据
    :return: 转变成列表或字典形式的数据
    """
    ...


def make_final_data(recorder: BaseRecorder, data: Iterable) -> Union[list, dict]:
    """将传入的一维数据转换为列表或字典形式，添加前后列数据
    :param recorder: BaseRecorder对象
    :param data: 要处理的数据
    :return: 转变成列表或字典形式的数据
    """
    ...


def get_csv(recorder: Recorder) -> Tuple[TextIOWrapper, bool]:
    """获取文件读写对象
    :param recorder: Recorder对象
    :return: (文件读写对象, 是否新文件)
    """
    ...


def get_wb(recorder: Recorder) -> Tuple[Workbook, bool]:
    """获取Workbook对象
    :param recorder: Recorder对象
    :return: (Workbook对象, 是否新文件)
    """
    ...


def get_ws(wb: Workbook, table: Optional[str], tables: List[str], new_file: bool) -> Tuple[Worksheet, bool]:
    """获取Worksheet对象
    :param wb: Workbook对象
    :param table: 表名，None代表活动表格
    :param tables: 工作簿所有表名组成的列表
    :param new_file: 是否新文件
    :return: (Worksheet对象, 是否新文件)
    """
    ...


def get_tables(path: Union[str, Path]) -> List[str]:
    """获取所有数据表名称
    :param path: 文件路径
    :return: 表名组成的列表
    """
    ...


def do_nothing(*args, **kwargs) -> None:
    """什么都不干"""
    ...


def get_key_cols(cols: Union[str, int, list, tuple, bool], header: Header, is_header: bool) -> List[int]:
    """获取作为关键字的列，可以是多列
    :param cols: 列号或列名，或它们组成的list或tuple
    :param header: Header格式
    :param is_header: cols中的str表示header还是列名
    :return: 列序号列表
    """
    ...


def is_sigal_data(data: Any) -> bool:
    """判断数据是否独立数据"""
    ...


def is_1D_data(data: Any) -> bool:
    """判断传入数据是否一维数据"""
    ...



# -*- coding:utf-8 -*-
from pathlib import Path

from openpyxl.reader.excel import load_workbook
from openpyxl.workbook import Workbook

from .cell_style import CellStyle
from .tools import (make_valid_name, make_final_data_simplify, make_final_data,
                    Header, ZeroHeader, process_content_xlsx, ok_list_str, data2ws_follow, data2ws, data2ws_style)


class OriginalSetter(object):
    def __init__(self, recorder):
        self._recorder = recorder

    def cache_size(self, size):
        if not isinstance(size, int) or size < 0:
            raise TypeError('cache_size值只能是int，且必须>=0')
        self._recorder._cache = size
        return self

    def path(self, path):
        if self._recorder._path:
            self._recorder.record()
        p = Path(path)
        self._recorder._path = str(p.parent / make_valid_name(p.name))
        self._recorder._file_exists = False
        return self

    def show_msg(self, on_off):
        self._recorder.show_msg = on_off
        return self

    def auto_backup(self, interval=None, path=None, new_name=None):
        if path is not None:
            self._recorder._backup_path = path
        if isinstance(new_name, bool):
            self._recorder._backup_new_name = new_name
        if interval is not None:
            self._recorder._backup_interval = interval
        return self


class BaseSetter(OriginalSetter):
    def table(self, name):
        self._recorder._table = name
        return self

    def auto_new_header(self, on_off=True):
        self._recorder.record()
        self._recorder._auto_new_header = on_off
        return self

    def before(self, data):
        return self._set_after_before(True, data)

    def after(self, data):
        return self._set_after_before(False, data)

    def _set_after_before(self, before, data):
        if isinstance(data, (list, dict)):
            data = data
        elif isinstance(data, tuple):
            data = list(data)
        elif data is not None:
            data = [data]
        setattr(self._recorder, '_before' if before else '_after', data)
        if self._recorder._after or self._recorder._before:
            self._recorder._make_final_data = make_final_data
        else:
            self._recorder._make_final_data = make_final_data_simplify
        return self


class RecorderSetter(BaseSetter):
    def encoding(self, encoding):
        self._recorder._encoding = encoding
        return self

    def header(self, header, table=None, to_file=True, row=None):
        if not header or not isinstance(header, (list, tuple)):
            raise ValueError('header不能为空且必须为list或tuple格式。')

        self._recorder.record()
        row = row or self._recorder._header_row.get(table, 1)
        with self._recorder._lock:
            header = Header(header)
            if self._recorder.type == 'xlsx':
                table = table or self._recorder.table
                self._recorder._header[table] = header
                if to_file:
                    set_xlsx_header(self._recorder, header, table, row)
            elif self._recorder.type == 'csv':
                self._recorder._header[None] = header
                if to_file:
                    set_csv_header(self._recorder, header, row)
            else:
                self._recorder._header[None] = header

        return self

    def header_row(self, num, table=None):
        if num < 0:
            raise ValueError('num不能小于0。')
        self._recorder._header_row[table] = num
        self._recorder._header[self._recorder.table] = ZeroHeader() if num == 0 else None
        return self

    def delimiter(self, delimiter):
        self._recorder._delimiter = delimiter
        return self

    def quote_char(self, quote_char):
        self._recorder._quote_char = quote_char
        return self

    def path(self, path, file_type=None):
        super().path(path)
        if not file_type:
            suffix = Path(path).suffix.lower()
            if suffix:
                file_type = suffix[1:]
        self.file_type(file_type)
        self._recorder._header = {None: None}
        self._recorder._header_row = {None: 1}
        return self

    def file_type(self, file_type):
        if file_type not in self._recorder._SUPPORTS:
            file_type = 'txt'
        self._recorder._type = file_type
        self._recorder._set_methods(file_type)
        if file_type != 'xlsx':
            self._recorder._table = None
        return self

    def table(self, name):
        if isinstance(name, bool):
            name = None
        self._recorder._table = name
        return self

    def follow_styles(self, on_off=True):
        self._recorder._follow_styles = on_off
        if on_off:
            self._recorder._styles = None
            self._recorder._row_height = None
            self._recorder._methods['data'] = data2ws_follow
        else:
            self._recorder._methods['data'] = data2ws
        return self

    def new_row_height(self, height):
        self._recorder._row_height = height
        if height is not None:
            self._recorder._follow_styles = False
            self._recorder._methods['data'] = data2ws_style
        else:
            self._recorder._methods['data'] = data2ws
        return self

    def new_row_styles(self, styles):
        self._recorder.record()
        self._recorder._styles = styles
        if styles is not None:
            self._recorder._follow_styles = False
            self._recorder._methods['data'] = data2ws_style
        else:
            self._recorder._methods['data'] = data2ws
        return self

    def data_col(self, col):
        if isinstance(col, int):
            self._recorder.data_col = col
        elif isinstance(col, str):
            self._recorder.data_col = ZeroHeader()._KEY_NUM.get(col.upper(), 1)
        elif col is None:
            self._recorder.data_col = 0
        else:
            raise TypeError('col值只能是int、str或None。')
        return self

    def link_style(self, style=True):
        if style is True:
            style = CellStyle()
            style.font.set_color("0000FF")
            style.font.set_underline('single')
        self._recorder._link_style = style
        return self


class DBSetter(BaseSetter):
    def path(self, path, table=None):
        with self._recorder._lock:
            super().path(path)
            if self._recorder._conn is not None:
                self._recorder._close_connection()
            self._recorder._connect()

            if table:
                self.table(table)
            else:
                r = self._recorder.run_sql("select name from sqlite_master where type='table'")
                self._recorder._table = r[0] if r else None

            self._recorder._data = {}
            self._recorder._close_connection()
        return self

    def table(self, name):
        if '`' in name:
            raise ValueError('table名称不能包含字符"`"。')
        self._recorder._table = name
        return self


def set_csv_header(recorder, header, row):
    if not recorder.path:
        raise FileNotFoundError('未指定文件。')
    from csv import writer
    if recorder._file_exists or Path(recorder.path).exists():
        with open(recorder.path, 'r', newline='', encoding=recorder._encoding) as f:
            lines = f.readlines()
            content1 = lines[:row - 1]
            content2 = lines[row:]

        with open(recorder.path, 'w', newline='', encoding=recorder._encoding) as f:
            f.write("".join(content1))
            csv_write = writer(f, delimiter=recorder._delimiter, quotechar=recorder._quote_char)
            con_len = len(content1)
            if con_len < row - 1:
                for _ in range(row - con_len - 1):
                    csv_write.writerow([])
            csv_write.writerow(ok_list_str(header))

        with open(recorder.path, 'a+', newline='', encoding=recorder._encoding) as f:
            f.write("".join(content2))

    else:
        Path(recorder.path).parent.mkdir(parents=True, exist_ok=True)
        with open(recorder.path, 'w', newline='', encoding=recorder._encoding) as f:
            csv_write = writer(f, delimiter=recorder._delimiter, quotechar=recorder._quote_char)
            for _ in range(row - 1):
                csv_write.writerow([])
            csv_write.writerow(ok_list_str(header))

    recorder._file_exists = True


def set_xlsx_header(recorder, header, table, row):
    if not recorder.path:
        raise FileNotFoundError('未指定文件。')
    if recorder._file_exists or Path(recorder.path).exists():
        wb = load_workbook(recorder.path)
        if table:
            ws = wb[table] if table in [i.title for i in wb.worksheets] else wb.create_sheet(title=table)
        else:
            ws = wb.active

    else:
        Path(recorder.path).parent.mkdir(parents=True, exist_ok=True)
        wb = Workbook()
        ws = wb.active
        if table:
            ws.title = table

    for c, i in header.items():
        ws.cell(row, c, value=process_content_xlsx(i))
    len_row = len(ws[row])
    len_header = len(header)
    if len_row > len_header:
        for c in range(len_header + 1, len_row + 1):
            ws.cell(row, c, value=None)

    wb.save(recorder.path)
    wb.close()
    recorder._file_exists = True



# -*- coding:utf-8 -*-
from pathlib import Path
from sqlite3 import Connection, Cursor
from typing import Union, Any, Optional

from .base import BaseRecorder
from .setter import DBSetter


class DBRecorder(BaseRecorder):
    _conn: Optional[Connection] = ...
    _cur: Optional[Cursor] = ...
    _setter: Optional[DBSetter] = ...
    _data: dict = ...
    data: dict = ...

    def __init__(self,
                 path: Union[str, Path] = None,
                 cache_size: int = 1000,
                 table: str = None):
        """用于存储数据到sqlite的工具
        :param path: 保存的文件路径
        :param cache_size: 每接收多少条记录写入文件，0为不自动写入
        :param table: 默认表名
        """
        ...

    @property
    def set(self) -> DBSetter:
        """返回用于设置属性的对象"""
        ...

    @property
    def tables(self) -> list:
        """返回所有表名"""
        ...

    def add_data(self, data: Any, table: str = None) -> None:
        """添加数据
        :param data: 可以是一维或二维数据，dict格式可向对应列填写数据，其余格式按顺序从左到右填入各列
        :param table: 数据要插入的表名称
        :return: None
        """
        ...

    def run_sql(self, sql: str, single: bool = True, commit: bool = False) -> Union[None, list, tuple]:
        """执行sql语句并返回结果
        :param sql: sql语句
        :param single: 是否只获取一个结果
        :param commit: 是否提交到数据库
        :return: 查找到的结果，没有结果时返回None
        """
        ...

    def _connect(self) -> None:
        """连接数据库"""
        ...

    def _close_connection(self) -> None:
        """关闭数据库 """
        ...

    def _record(self) -> None:
        """保存数据到sqlite"""
        ...

    def _to_database(self,
                     data_list: list,
                     table: str,
                     tables: dict) -> None:
        """把数据批量写入指定数据表
        :param data_list: 要写入的数据组成的列表
        :param table: 要写入数据的数据表名称
        :param tables: 数据库中数据表和列信息
        :return: None
        """
        ...

    def _handle_data(self, data: Any) -> list:
        """接收数据后的格式化"""
        ...
